@@ -22,7 +22,7 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.28 $ $Date: 2004/05/06 19:41:58 $
+ * CVS: $Revision: 1.30 $ $Date: 2004/11/03 16:35:31 $
  */ 
 #ifdef ALPHA
 #include "defs.h"
@@ -704,9 +704,10 @@ alpha_dump_line_number(char *name, ulong callpc)
 try_closest:
 	get_line_number(callpc, buf, FALSE);
 	if (strlen(buf)) {
-	if (retries) {
-		p = strstr(buf, ": ");
-			*p = NULLCHAR;
+		if (retries) {
+			p = strstr(buf, ": ");
+			if (p)
+				*p = NULLCHAR;
 		} 
 		fprintf(fp, "    %s\n", buf);
 	} else {
@@ -798,7 +799,7 @@ alpha_frame_offset(struct gnu_request *req, ulong alt_pc)
 	}
 
 use_gdb:
-#if defined(GDB_6_1)
+#if defined(GDB_6_0) || defined(GDB_6_1)
 {
 	static int gdb_frame_offset_warnings = 10;
 
