@@ -14,7 +14,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * CVS: $Revision: 1.25 $ $Date: 2004/06/21 17:48:18 $
+ * CVS: $Revision: 1.28 $ $Date: 2004/11/08 17:01:23 $
  */
 
 /*
@@ -112,7 +112,7 @@ int verify_rpm_targets(void);
 #define TARGET_CFLAGS_IA64   "TARGET_CFLAGS="
 #define TARGET_CFLAGS_S390   "TARGET_CFLAGS=-D_FILE_OFFSET_BITS=64"
 #define TARGET_CFLAGS_S390X  "TARGET_CFLAGS="
-#define TARGET_CFLAGS_PPC64  "TARGET_CFLAGS="
+#define TARGET_CFLAGS_PPC64  "TARGET_CFLAGS=-m64"
 #define TARGET_CFLAGS_X86_64 "TARGET_CFLAGS="
 
 /*
@@ -120,15 +120,25 @@ int verify_rpm_targets(void);
  *  of gdb for a given architecture.  But for practical purposes,
  *  it's a one-size-fits-all scheme, and they all use the default.
  */
-#define LAST_GDB  	      "GDB=gdb-5.3post-0.20021129.36rh"
-#define LAST_GDB_VERSION_IN   "Red Hat Linux (5.3post-0.20021129.36rh)"
-#define LAST_GDB_FILES        "GDB_FILES=${GDB_5.3post-0.20021129.36rh_FILES}"
-#define LAST_GDB_OFILES       "GDB_OFILES=${GDB_5.3post-0.20021129.36rh_OFILES}"
+// #define DEFAULT_GDB  	    "GDB=gdb-5.3post-0.20021129.36rh"
+// #define DEFAULT_GDB_VERSION_IN   "Red Hat Linux (5.3post-0.20021129.36rh)"
+// #define DEFAULT_GDB_FILES        "GDB_FILES=${GDB_5.3post-0.20021129.36rh_FILES}"
+// #define DEFAULT_GDB_OFILES       "GDB_OFILES=${GDB_5.3post-0.20021129.36rh_OFILES}"
 
-#define DEFAULT_GDB             "GDB=gdb-6.1"
-#define DEFAULT_GDB_VERSION_IN  "6.1"
-#define DEFAULT_GDB_FILES       "GDB_FILES=${GDB_6.1_FILES}"
-#define DEFAULT_GDB_OFILES      "GDB_OFILES=${GDB_6.1_OFILES}"
+/*
+ *  NOTE: Reverted from gdb-6.1 back to gdb-6.0 in order to deal 
+ *  with gdb-6.1's inability to properly access all debug data
+ *  from vmlinux files built with gcc-3.4.*.
+ */
+// #define DEFAULT_GDB             "GDB=gdb-6.1"
+// #define DEFAULT_GDB_VERSION_IN  "6.1"
+// #define DEFAULT_GDB_FILES       "GDB_FILES=${GDB_6.1_FILES}"
+// #define DEFAULT_GDB_OFILES      "GDB_OFILES=${GDB_6.1_OFILES}"
+
+#define DEFAULT_GDB             "GDB=gdb-6.0"
+#define DEFAULT_GDB_VERSION_IN  "6.0"
+#define DEFAULT_GDB_FILES       "GDB_FILES=${GDB_6.0_FILES}"
+#define DEFAULT_GDB_OFILES      "GDB_OFILES=${GDB_6.0_OFILES}"
 
 #define GDB_X86      		DEFAULT_GDB
 #define GDB_ALPHA    		DEFAULT_GDB
@@ -1065,7 +1075,7 @@ make_spec_file(void)
 	printf("Vendor: Red Hat, Inc.\n");
 	printf("Packager: Dave Anderson <anderson@redhat.com>\n");
 	printf("ExclusiveOS: Linux\n");
-	printf("ExclusiveArch: i386 alpha ia64 ppc ppc64 x86_64\n");
+	printf("ExclusiveArch: i386 alpha ia64 ppc ppc64 ppc64pseries ppc64iseries x86_64\n");
 	printf("Buildroot: %%{_tmppath}/%%{name}-root\n");
 	printf("BuildRequires: ncurses-devel zlib-devel\n");
 	printf("# Patch0: crash-3.3-20.installfix.patch (patch example)\n");
