@@ -27,7 +27,7 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.31 $ $Date: 2004/06/22 16:43:33 $
+ * CVS: $Revision: 1.36 $ $Date: 2004/11/10 21:02:28 $
  */
 
 #include "defs.h"
@@ -38,7 +38,8 @@ static int find_booted_kernel(void);
 static int find_booted_system_map(void);
 static int verify_utsname(char *);
 static char **build_searchdirs(int, int *);
-static int redhat_kernel_directory(char *);
+static int redhat_kernel_directory_v1(char *);
+static int redhat_kernel_directory_v2(char *);
 static int redhat_debug_directory(char *);
 static int file_dump(ulong, ulong, ulong, int, int);
 static ulong *create_dentry_array(ulong, int *);
@@ -311,10 +312,11 @@ build_searchdirs(int create, int *preferred)
 		*preferred = 0;
 
 	/*
-	 *  Allow, at a minimum, the defaults plus an extra two directories for
-	 *  the /usr/src/redhat/BUILD/kernel-xxx and Red Hat debug directories.
+	 *  Allow, at a minimum, the defaults plus an extra three directories 
+	 *  for the two possible /usr/src/redhat/BUILD/kernel-xxx locations 
+	 *  plus the Red Hat debug directory.
 	 */  
-	cnt = DEFAULT_SEARCHDIRS + 2;  
+	cnt = DEFAULT_SEARCHDIRS + 3;  
 
         if ((dirp = opendir("/usr/src"))) {
                 for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) 
@@ -364,7 +366,7 @@ build_searchdirs(int create, int *preferred)
 		searchdirs[cnt] = NULL;
 	}
 
-        if (redhat_kernel_directory(dirbuf)) {
+        if (redhat_kernel_directory_v1(dirbuf)) {
                 if ((searchdirs[cnt] = (char *) 
 		    malloc(strlen(dirbuf)+2)) == NULL) {
                         error(INFO, 
@@ -376,6 +378,18 @@ build_searchdirs(int create, int *preferred)
                 }
         }
 
+        if (redhat_kernel_directory_v2(dirbuf)) {
+                if ((searchdirs[cnt] = (char *)
+                    malloc(strlen(dirbuf)+2)) == NULL) {
+                        error(INFO,
+                            "/usr/src/redhat directory entry malloc: %s\n",
+                                strerror(errno));
+                } else {
+                        sprintf(searchdirs[cnt], "%s/", dirbuf);
+                        cnt++;
+                }
+        }
+
         if (redhat_debug_directory(dirbuf)) {
                 if ((searchdirs[cnt] = (char *)
                      malloc(strlen(dirbuf)+2)) == NULL) {
@@ -409,7 +423,7 @@ build_searchdirs(int create, int *preferred)
 }
 
 static int
-redhat_kernel_directory(char *buf)
+redhat_kernel_directory_v1(char *buf)
 {
 	char *p1, *p2;
 
@@ -429,6 +443,35 @@ redhat_kernel_directory(char *buf)
 	return TRUE;
 }
 
+static int
+redhat_kernel_directory_v2(char *buf)
+{
+        char *p1, *p2;
+
+        if (!strstr(kt->proc_version, "Linux version "))
+                return FALSE;
+
+        BZERO(buf, BUFSIZE);
+        sprintf(buf, "/usr/src/redhat/BUILD/kernel-");
+
+        p1 = &kt->proc_version[strlen("Linux version ")];
+        p2 = &buf[strlen(buf)];
+
+        while (((*p1 >= '0') && (*p1 <= '9')) || (*p1 == '.'))
+                *p2++ = *p1++;
+
+        strcat(buf, "/linux-");
+
+        p1 = &kt->proc_version[strlen("Linux version ")];
+        p2 = &buf[strlen(buf)];
+
+        while (((*p1 >= '0') && (*p1 <= '9')) || (*p1 == '.'))
+                *p2++ = *p1++;
+
+        return TRUE;
+}
+
+
 static int
 redhat_debug_directory(char *buf)
 {
@@ -504,6 +547,9 @@ find_booted_kernel(void)
 		if (!dirp)
 			continue;
 	        for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
+			if (dp->d_name[0] == '.')
+				continue;
+
 			sprintf(kernel, "%s%s", searchdirs[i], dp->d_name);
 
 			if (mount_point(kernel) ||
@@ -687,7 +733,7 @@ find_booted_system_map(void)
 {
 	char system_map[BUFSIZE];
 	char **searchdirs;
-	int i, fd;
+	int i;
         DIR *dirp;
         struct dirent *dp;
 	int found;
@@ -746,8 +792,6 @@ find_booted_system_map(void)
 		build_searchdirs(DESTROY, NULL);
 	}
 
-	close(fd);
-
 	if (found) {
                 if ((pc->system_map = (char *)malloc
 		    (strlen(system_map)+1)) == NULL) 
@@ -922,7 +966,8 @@ search_directory_tree(char *directory, char *file)
 
 	if (!file_exists("/usr/bin/find", NULL) || 
 	    !file_exists("/bin/echo", NULL) ||
-	    !is_directory(directory)) 
+	    !is_directory(directory) ||
+	    (*file == '(')) 
 		return NULL;
 
 	sprintf(command, 
@@ -1585,7 +1630,7 @@ inode_type(char *inode_buf, char *pathname)
 		} else {
 			if (symbol_exists("rdwr_pipe_fops") && 
 			    (i_fop_off = OFFSET(inode_i_fop)) > 0) {
-				 inode_i_op = ULONG(inode_buf + i_fop_off);
+				 inode_i_fop = ULONG(inode_buf + i_fop_off);
 				 if (inode_i_fop == 
 				     symbol_value("rdwr_pipe_fops")) { 
 					type = "PIPE";
@@ -1677,6 +1722,7 @@ create_dentry_array(ulong list_addr, int *count)
 void
 vfs_init(void)
 { 
+        MEMBER_OFFSET_INIT(nlm_file_f_file, "nlm_file", "f_file");
 	MEMBER_OFFSET_INIT(task_struct_files, "task_struct", "files");
 	MEMBER_OFFSET_INIT(task_struct_fs, "task_struct", "fs");
 	MEMBER_OFFSET_INIT(fs_struct_root, "fs_struct", "root");
@@ -1723,7 +1769,6 @@ vfs_init(void)
         MEMBER_OFFSET_INIT(super_block_s_type, "super_block", "s_type");
         MEMBER_OFFSET_INIT(file_system_type_name, "file_system_type", "name");
 	MEMBER_OFFSET_INIT(super_block_s_files, "super_block", "s_files");
-        MEMBER_OFFSET_INIT(nlm_file_f_file, "nlm_file", "f_file");
         MEMBER_OFFSET_INIT(inode_i_flock, "inode", "i_flock");
         MEMBER_OFFSET_INIT(file_lock_fl_owner, "file_lock", "fl_owner");
         MEMBER_OFFSET_INIT(nlm_host_h_exportent, "nlm_host", "h_exportent");
@@ -3257,10 +3302,7 @@ create_memory_device(dev_t dev)
 				pc->live_memsrc, major(stat.st_rdev), 
 				minor(stat.st_rdev));
 		else 
-			error(FATAL, 
-			    "%s: invalid mode: %o -- please delete it!\n",
-				pc->live_memsrc,
-				stat.st_mode & (~(S_IFCHR)));
+			unlink(pc->live_memsrc);
 	} 
 
 	/* 
