@@ -1,7 +1,7 @@
 /* Read ELF (Executable and Linking Format) object files for GDB.
 
    Copyright 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-   2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+   2000, 2001, 2002, 2003 Free Software Foundation, Inc.
    Portions Copyright (C) 2001, 2002 Mission Critical Linux, Inc.
    Copyright (c) 2002, 2003, 2004 Red Hat, Inc. All rights reserved.
 
@@ -24,7 +24,7 @@
    Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
-/* CVS: $Revision: 1.2 $ $Date: 2004/04/30 19:40:56 $ */
+/* CVS: $Revision: 1.2 $ $Date: 2004/11/03 20:00:11 $ */
 
 #include "defs.h"
 #include "bfd.h"
@@ -81,43 +81,72 @@ static void free_elfinfo (void *);
 static void
 elf_locate_sections (bfd *ignore_abfd, asection *sectp, void *eip)
 {
-  struct elfinfo *ei;
+  register struct elfinfo *ei;
 
   ei = (struct elfinfo *) eip;
-  if (strcmp (sectp->name, ".debug") == 0)
+  if (STREQ (sectp->name, ".debug"))
     {
       ei->dboffset = sectp->filepos;
       ei->dbsize = bfd_get_section_size_before_reloc (sectp);
     }
-  else if (strcmp (sectp->name, ".line") == 0)
+  else if (STREQ (sectp->name, ".line"))
     {
       ei->lnoffset = sectp->filepos;
       ei->lnsize = bfd_get_section_size_before_reloc (sectp);
     }
-  else if (strcmp (sectp->name, ".stab") == 0)
+  else if (STREQ (sectp->name, ".stab"))
     {
       ei->stabsect = sectp;
     }
-  else if (strcmp (sectp->name, ".stab.index") == 0)
+  else if (STREQ (sectp->name, ".stab.index"))
     {
       ei->stabindexsect = sectp;
     }
-  else if (strcmp (sectp->name, ".mdebug") == 0)
+  else if (STREQ (sectp->name, ".mdebug"))
     {
       ei->mdebugsect = sectp;
     }
 }
 
+#if 0				/* Currently unused */
+
+char *
+elf_interpreter (bfd *abfd)
+{
+  sec_ptr interp_sec;
+  unsigned size;
+  char *interp = NULL;
+
+  interp_sec = bfd_get_section_by_name (abfd, ".interp");
+  if (interp_sec)
+    {
+      size = bfd_section_size (abfd, interp_sec);
+      interp = alloca (size);
+      if (bfd_get_section_contents (abfd, interp_sec, interp, (file_ptr) 0,
+				    size))
+	{
+	  interp = savestring (interp, size - 1);
+	}
+      else
+	{
+	  interp = NULL;
+	}
+    }
+  return (interp);
+}
+
+#endif
+
 static struct minimal_symbol *
-record_minimal_symbol (char *name, CORE_ADDR address,
-		       enum minimal_symbol_type ms_type,
-		       asection *bfd_section, struct objfile *objfile)
+record_minimal_symbol_and_info (char *name, CORE_ADDR address,
+				enum minimal_symbol_type ms_type, char *info,	/* FIXME, is this really char *? */
+				asection *bfd_section, struct objfile *objfile)
 {
   if (ms_type == mst_text || ms_type == mst_file_text)
     address = SMASH_TEXT_ADDRESS (address);
 
   return prim_record_minimal_symbol_and_info
-    (name, address, ms_type, NULL, bfd_section->index, bfd_section, objfile);
+    (name, address, ms_type, info, bfd_section->index, bfd_section, objfile);
 }
 
 /*
@@ -163,10 +192,11 @@ elf_symtab_read (struct objfile *objfile, int dynamic)
      seen any section info for it yet.  */
   asymbol *filesym = 0;
 #ifdef SOFUN_ADDRESS_MAYBE_MISSING
-  /* Name of filesym, as saved on the objfile_obstack.  */
-  char *filesymname = obsavestring ("", 0, &objfile->objfile_obstack);
+  /* Name of filesym, as saved on the symbol_obstack.  */
+  char *filesymname = obsavestring ("", 0, &objfile->symbol_obstack);
 #endif
   struct dbx_symfile_info *dbx = objfile->sym_stab_info;
+  unsigned long size;
   int stripped = (bfd_get_symcount (objfile->obfd) == 0);
 
   if (dynamic)
@@ -226,9 +256,9 @@ elf_symtab_read (struct objfile *objfile, int dynamic)
 	      if (symaddr == 0)
 		continue;
 	      symaddr += offset;
-	      msym = record_minimal_symbol
+	      msym = record_minimal_symbol_and_info
 		((char *) sym->name, symaddr,
-		 mst_solib_trampoline, sym->section, objfile);
+		 mst_solib_trampoline, NULL, sym->section, objfile);
 #ifdef SOFUN_ADDRESS_MAYBE_MISSING
 	      if (msym != NULL)
 		msym->filename = filesymname;
@@ -255,7 +285,7 @@ elf_symtab_read (struct objfile *objfile, int dynamic)
 #ifdef SOFUN_ADDRESS_MAYBE_MISSING
 	      filesymname =
 		obsavestring ((char *) filesym->name, strlen (filesym->name),
-			      &objfile->objfile_obstack);
+			      &objfile->symbol_obstack);
 #endif
 	    }
 	  else if (sym->flags & (BSF_GLOBAL | BSF_LOCAL | BSF_WEAK))
@@ -346,7 +376,10 @@ elf_symtab_read (struct objfile *objfile, int dynamic)
 		  else if (sym->flags & BSF_LOCAL)
 		    {
 		      /* Named Local variable in a Data section.
-		         Check its name for stabs-in-elf.  */
+		         Check its name for stabs-in-elf.  The STREQ
+		         macro checks the first character inline, so
+		         we only actually do a strcmp function call on
+		         names that start with 'B' or 'D'.  */
 		      int special_local_sect;
 		      if (strcmp ("Bbss.bss", sym->name) == 0)
 			special_local_sect = SECT_OFF_BSS (objfile);
@@ -369,17 +402,9 @@ elf_symtab_read (struct objfile *objfile, int dynamic)
 				= max (SECT_OFF_BSS (objfile),
 				       max (SECT_OFF_DATA (objfile),
 					    SECT_OFF_RODATA (objfile)));
-
-                              /* max_index is the largest index we'll
-                                 use into this array, so we must
-                                 allocate max_index+1 elements for it.
-                                 However, 'struct stab_section_info'
-                                 already includes one element, so we
-                                 need to allocate max_index aadditional
-                                 elements.  */
 			      size = (sizeof (struct stab_section_info) 
 				      + (sizeof (CORE_ADDR)
-					 * max_index));
+					 * (max_index - 1)));
 			      sectinfo = (struct stab_section_info *)
 				xmmalloc (objfile->md, size);
 			      memset (sectinfo, 0, size);
@@ -436,15 +461,11 @@ elf_symtab_read (struct objfile *objfile, int dynamic)
 		  /* ms_type = mst_unknown; */
 		  continue;	/* Skip this symbol. */
 		}
-	      msym = record_minimal_symbol
+	      /* Pass symbol size field in via BFD.  FIXME!!!  */
+	      size = ((elf_symbol_type *) sym)->internal_elf_sym.st_size;
+	      msym = record_minimal_symbol_and_info
 		((char *) sym->name, symaddr,
-		 ms_type, sym->section, objfile);
-	      if (msym)
-	      {
-		/* Pass symbol size field in via BFD.  FIXME!!!  */
-		unsigned long size = ((elf_symbol_type *) sym)->internal_elf_sym.st_size;
-		MSYMBOL_SIZE(msym) = size;
-	      }
+		 ms_type, (void *) size, sym->section, objfile);
 #ifdef CRASH_MERGE
               {
                 void patch_load_module(struct objfile *,
@@ -600,9 +621,8 @@ elf_symfile_read (struct objfile *objfile, int mainline)
 			    ei.lnoffset, ei.lnsize);
     }
 
-  /* FIXME: kettenis/20030504: This still needs to be integrated with
-     dwarf2read.c in a better way.  */
-  dwarf2_build_frame_info (objfile);
+  if (DWARF2_BUILD_FRAME_INFO_P ())
+    DWARF2_BUILD_FRAME_INFO(objfile);
 }
 
 /* This cleans up the objfile's sym_stab_info pointer, and the chain of
@@ -701,7 +721,7 @@ elfstab_offset_sections (struct objfile *objfile, struct partial_symtab *pst)
   for (; maybe; maybe = maybe->next)
     {
       if (filename[0] == maybe->filename[0]
-	  && strcmp (filename, maybe->filename) == 0)
+	  && STREQ (filename, maybe->filename))
 	{
 	  /* We found a match.  But there might be several source files
 	     (from different directories) with the same name.  */
@@ -723,7 +743,7 @@ elfstab_offset_sections (struct objfile *objfile, struct partial_symtab *pst)
       /* Found it!  Allocate a new psymtab struct, and fill it in.  */
       maybe->found++;
       pst->section_offsets = (struct section_offsets *)
-	obstack_alloc (&objfile->objfile_obstack, 
+	obstack_alloc (&objfile->psymbol_obstack, 
 		       SIZEOF_N_SECTION_OFFSETS (objfile->num_sections));
       for (i = 0; i < maybe->num_sections; i++)
 	(pst->section_offsets)->offsets[i] = maybe->sections[i];
