@@ -1,7 +1,7 @@
 /* Top level stuff for GDB, the GNU debugger.
 
    Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
-   1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Free Software
+   1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003 Free Software
    Foundation, Inc.
    Portions Copyright (C) 2001, 2002 Mission Critical Linux, Inc.
    Copyright (c) 2002, 2003, 2004 Red Hat, Inc. All rights reserved.
@@ -23,7 +23,7 @@
    Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
-/* CVS: $Revision: 1.2 $ $Date: 2004/04/30 19:40:56 $ */
+/* CVS: $Revision: 1.2 $ $Date: 2004/11/03 20:00:11 $ */
 
 #include "defs.h"
 #include "top.h"
@@ -76,10 +76,10 @@ char *gdb_sysroot = 0;
 struct ui_file *gdb_stdout;
 struct ui_file *gdb_stderr;
 struct ui_file *gdb_stdlog;
+struct ui_file *gdb_stdtarg;
 struct ui_file *gdb_stdin;
 /* target IO streams */
 struct ui_file *gdb_stdtargin;
-struct ui_file *gdb_stdtarg;
 struct ui_file *gdb_stdtargerr;
 
 /* Whether to enable writing into executable and core files */
@@ -154,7 +154,7 @@ captured_main (void *data)
   struct stat homebuf, cwdbuf;
   char *homedir, *homeinit;
 
-  int i;
+  register int i;
 
   long time_at_startup = get_run_time ();
 
@@ -167,6 +167,8 @@ captured_main (void *data)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  START_PROGRESS (argv[0], 0);
+
   /* This needs to happen before the first use of malloc.  */
   init_malloc (NULL);
 
@@ -266,6 +268,8 @@ captured_main (void *data)
       {"dbx", no_argument, &dbx_commands, 1},
       {"readnow", no_argument, &readnow_symbol_files, 1},
       {"r", no_argument, &readnow_symbol_files, 1},
+      {"mapped", no_argument, &mapped_symbol_files, 1},
+      {"m", no_argument, &mapped_symbol_files, 1},
       {"quiet", no_argument, &quiet, 1},
       {"q", no_argument, &quiet, 1},
       {"silent", no_argument, &quiet, 1},
@@ -758,6 +762,8 @@ extern int gdbtk_test (char *);
   BEFORE_MAIN_LOOP_HOOK;
 #endif
 
+  END_PROGRESS (argv[0]);
+
   /* Show time and/or space usage.  */
 
   if (display_time)
