@@ -1,8 +1,9 @@
 /* General utility routines for GDB, the GNU debugger.
 
    Copyright 1986, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995,
-   1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Free Software
+   1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003 Free Software
    Foundation, Inc.
+
    Portions Copyright (C) 2001, 2002 Mission Critical Linux, Inc.
    Copyright (c) 2002, 2003, 2004 Red Hat, Inc. All rights reserved.
 
@@ -23,7 +24,7 @@
    Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
-/* CVS: $Revision: 1.2 $ $Date: 2004/04/30 19:40:57 $ */
+/* CVS: $Revision: 1.2 $ $Date: 2004/11/03 20:00:11 $ */
 
 #include "defs.h"
 #include "gdb_assert.h"
@@ -31,10 +32,6 @@
 #include "gdb_string.h"
 #include "event-top.h"
 
-#ifdef TUI
-#include "tui/tui.h"		/* For tui_get_command_dimension.   */
-#endif
-
 #ifdef __GO32__
 #include <pc.h>
 #endif
@@ -67,7 +64,11 @@
 #include <term.h>
 #endif
 
-#include "readline/readline.h"
+#include <readline/readline.h>
+
+#ifdef USE_MMALLOC
+#include "mmalloc.h"
+#endif
 
 #ifdef NEED_DECLARATION_MALLOC
 extern PTR malloc ();		/* OK: PTR */
@@ -100,11 +101,14 @@ static void vfprintf_maybe_filtered (struct ui_file *, const char *,
 
 static void fputs_maybe_filtered (const char *, struct ui_file *, int);
 
-static void do_my_cleanups (struct cleanup **, struct cleanup *);
+#if defined (USE_MMALLOC) && !defined (NO_MMCHECK)
+static void malloc_botch (void);
+#endif
 
 static void prompt_for_continue (void);
 
-static void set_screen_size (void);
+static void set_width_command (char *, int, struct cmd_list_element *);
+
 static void set_width (void);
 
 /* Chain of cleanup actions established with make_cleanup,
@@ -268,9 +272,9 @@ struct cleanup *
 make_my_cleanup (struct cleanup **pmy_chain, make_cleanup_ftype *function,
 		 void *arg)
 {
-  struct cleanup *new
+  register struct cleanup *new
     = (struct cleanup *) xmalloc (sizeof (struct cleanup));
-  struct cleanup *old_chain = *pmy_chain;
+  register struct cleanup *old_chain = *pmy_chain;
 
   new->next = *pmy_chain;
   new->function = function;
@@ -284,40 +288,40 @@ make_my_cleanup (struct cleanup **pmy_chain, make_cleanup_ftype *function,
    until we get back to the point OLD_CHAIN in the cleanup_chain.  */
 
 void
-do_cleanups (struct cleanup *old_chain)
+do_cleanups (register struct cleanup *old_chain)
 {
   do_my_cleanups (&cleanup_chain, old_chain);
 }
 
 void
-do_final_cleanups (struct cleanup *old_chain)
+do_final_cleanups (register struct cleanup *old_chain)
 {
   do_my_cleanups (&final_cleanup_chain, old_chain);
 }
 
 void
-do_run_cleanups (struct cleanup *old_chain)
+do_run_cleanups (register struct cleanup *old_chain)
 {
   do_my_cleanups (&run_cleanup_chain, old_chain);
 }
 
 void
-do_exec_cleanups (struct cleanup *old_chain)
+do_exec_cleanups (register struct cleanup *old_chain)
 {
   do_my_cleanups (&exec_cleanup_chain, old_chain);
 }
 
 void
-do_exec_error_cleanups (struct cleanup *old_chain)
+do_exec_error_cleanups (register struct cleanup *old_chain)
 {
   do_my_cleanups (&exec_error_cleanup_chain, old_chain);
 }
 
-static void
-do_my_cleanups (struct cleanup **pmy_chain,
-		struct cleanup *old_chain)
+void
+do_my_cleanups (register struct cleanup **pmy_chain,
+		register struct cleanup *old_chain)
 {
-  struct cleanup *ptr;
+  register struct cleanup *ptr;
   while ((ptr = *pmy_chain) != old_chain)
     {
       *pmy_chain = ptr->next;	/* Do this first incase recursion */
@@ -330,28 +334,28 @@ do_my_cleanups (struct cleanup **pmy_chain,
    until we get back to the point OLD_CHAIN in the cleanup_chain.  */
 
 void
-discard_cleanups (struct cleanup *old_chain)
+discard_cleanups (register struct cleanup *old_chain)
 {
   discard_my_cleanups (&cleanup_chain, old_chain);
 }
 
 void
-discard_final_cleanups (struct cleanup *old_chain)
+discard_final_cleanups (register struct cleanup *old_chain)
 {
   discard_my_cleanups (&final_cleanup_chain, old_chain);
 }
 
 void
-discard_exec_error_cleanups (struct cleanup *old_chain)
+discard_exec_error_cleanups (register struct cleanup *old_chain)
 {
   discard_my_cleanups (&exec_error_cleanup_chain, old_chain);
 }
 
 void
-discard_my_cleanups (struct cleanup **pmy_chain,
-		     struct cleanup *old_chain)
+discard_my_cleanups (register struct cleanup **pmy_chain,
+		     register struct cleanup *old_chain)
 {
-  struct cleanup *ptr;
+  register struct cleanup *ptr;
   while ((ptr = *pmy_chain) != old_chain)
     {
       *pmy_chain = ptr->next;
@@ -429,6 +433,7 @@ free_current_contents (void *ptr)
    In such cases, we may not be certain where the first cleanup is, unless
    we have a do-nothing one to always use as the base. */
 
+/* ARGSUSED */
 void
 null_cleanup (void *arg)
 {
@@ -578,7 +583,7 @@ vwarning (const char *string, va_list args)
       wrap_here ("");		/* Force out any buffered output */
       gdb_flush (gdb_stdout);
       if (warning_pre_print)
-	fputs_unfiltered (warning_pre_print, gdb_stderr);
+	fprintf_unfiltered (gdb_stderr, warning_pre_print);
       vfprintf_unfiltered (gdb_stderr, string, args);
       fprintf_unfiltered (gdb_stderr, "\n");
       va_end (args);
@@ -628,38 +633,6 @@ do_write (void *data, const char *buffer, long length_buffer)
   ui_file_write (data, buffer, length_buffer);
 }
 
-/* Cause a silent error to occur.  Any error message is recorded
-   though it is not issued.  */
-NORETURN void
-error_silent (const char *string, ...)
-{
-  va_list args;
-  struct ui_file *tmp_stream = mem_fileopen ();
-  va_start (args, string);
-  make_cleanup_ui_file_delete (tmp_stream);
-  vfprintf_unfiltered (tmp_stream, string, args);
-  /* Copy the stream into the GDB_LASTERR buffer.  */
-  ui_file_rewind (gdb_lasterr);
-  ui_file_put (tmp_stream, do_write, gdb_lasterr);
-  va_end (args);
-
-  throw_exception (RETURN_ERROR);
-}
-
-/* Output an error message including any pre-print text to gdb_stderr.  */
-void
-error_output_message (char *pre_print, char *msg)
-{
-  target_terminal_ours ();
-  wrap_here ("");		/* Force out any buffered output */
-  gdb_flush (gdb_stdout);
-  annotate_error_begin ();
-  if (pre_print)
-    fputs_filtered (pre_print, gdb_stderr);
-  fputs_filtered (msg, gdb_stderr);
-  fprintf_filtered (gdb_stderr, "\n");
-}
-
 NORETURN void
 error_stream (struct ui_file *stream)
 {
@@ -676,7 +649,7 @@ error_stream (struct ui_file *stream)
   gdb_flush (gdb_stdout);
   annotate_error_begin ();
   if (error_pre_print)
-    fputs_filtered (error_pre_print, gdb_stderr);
+    fprintf_filtered (gdb_stderr, error_pre_print);
   ui_file_put (stream, do_write, gdb_stderr);
   fprintf_filtered (gdb_stderr, "\n");
 #ifdef CRASH_MERGE
@@ -961,7 +934,7 @@ quit (void)
 
   /* Don't use *_filtered; we don't want to prompt the user to continue.  */
   if (quit_pre_print)
-    fputs_unfiltered (quit_pre_print, gdb_stderr);
+    fprintf_unfiltered (gdb_stderr, quit_pre_print);
 
 #ifdef __MSDOS__
   /* No steenking SIGINT will ever be coming our way when the
@@ -990,12 +963,18 @@ request_quit (int signo)
      about USG defines and stuff like that.  */
   signal (signo, request_quit);
 
+#ifdef REQUEST_QUIT
+  REQUEST_QUIT;
+#else
   if (immediate_quit)
     quit ();
+#endif
 }
 
 /* Memory management stuff (malloc friends).  */
 
+#if !defined (USE_MMALLOC)
+
 static void *
 mmalloc (void *md, size_t size)
 {
@@ -1023,13 +1002,63 @@ mfree (void *md, void *ptr)
   free (ptr);			/* NOTE: GDB's only call to free() */
 }
 
-/* This used to do something interesting with USE_MMALLOC.
- * It can be retired any time.  -- chastain 2004-01-19.  */
+#endif /* USE_MMALLOC */
+
+#if !defined (USE_MMALLOC) || defined (NO_MMCHECK)
+
 void
 init_malloc (void *md)
 {
 }
 
+#else /* Have mmalloc and want corruption checking */
+
+static void
+malloc_botch (void)
+{
+  fprintf_unfiltered (gdb_stderr, "Memory corruption\n");
+  internal_error (__FILE__, __LINE__, "failed internal consistency check");
+}
+
+/* Attempt to install hooks in mmalloc/mrealloc/mfree for the heap specified
+   by MD, to detect memory corruption.  Note that MD may be NULL to specify
+   the default heap that grows via sbrk.
+
+   Note that for freshly created regions, we must call mmcheckf prior to any
+   mallocs in the region.  Otherwise, any region which was allocated prior to
+   installing the checking hooks, which is later reallocated or freed, will
+   fail the checks!  The mmcheck function only allows initial hooks to be
+   installed before the first mmalloc.  However, anytime after we have called
+   mmcheck the first time to install the checking hooks, we can call it again
+   to update the function pointer to the memory corruption handler.
+
+   Returns zero on failure, non-zero on success. */
+
+#ifndef MMCHECK_FORCE
+#define MMCHECK_FORCE 0
+#endif
+
+void
+init_malloc (void *md)
+{
+  if (!mmcheckf (md, malloc_botch, MMCHECK_FORCE))
+    {
+      /* Don't use warning(), which relies on current_target being set
+         to something other than dummy_target, until after
+         initialize_all_files(). */
+
+      fprintf_unfiltered
+	(gdb_stderr,
+	 "warning: failed to install memory consistency checks; ");
+      fprintf_unfiltered (gdb_stderr,
+			  "configuration should define NO_MMCHECK or MMCHECK_FORCE\n");
+    }
+
+  mmtrace ();
+}
+
+#endif /* Have mmalloc and want corruption checking  */
+
 /* Called when a memory allocation fails, with the number of bytes of
    memory requested in SIZE. */
 
@@ -1161,17 +1190,6 @@ xfree (void *ptr)
 /* Like asprintf/vasprintf but get an internal_error if the call
    fails. */
 
-char *
-xstrprintf (const char *format, ...)
-{
-  char *ret;
-  va_list args;
-  va_start (args, format);
-  xvasprintf (&ret, format, args);
-  va_end (args);
-  return ret;
-}
-
 void
 xasprintf (char **ret, const char *format, ...)
 {
@@ -1204,7 +1222,7 @@ xvasprintf (char **ret, const char *format, va_list ap)
 int
 myread (int desc, char *addr, int len)
 {
-  int val;
+  register int val;
   int orglen = len;
 
   while (len > 0)
@@ -1227,7 +1245,7 @@ myread (int desc, char *addr, int len)
 char *
 savestring (const char *ptr, size_t size)
 {
-  char *p = (char *) xmalloc (size + 1);
+  register char *p = (char *) xmalloc (size + 1);
   memcpy (p, ptr, size);
   p[size] = 0;
   return p;
@@ -1236,7 +1254,7 @@ savestring (const char *ptr, size_t size)
 char *
 msavestring (void *md, const char *ptr, size_t size)
 {
-  char *p = (char *) xmmalloc (md, size + 1);
+  register char *p = (char *) xmmalloc (md, size + 1);
   memcpy (p, ptr, size);
   p[size] = 0;
   return p;
@@ -1249,7 +1267,7 @@ mstrsave (void *md, const char *ptr)
 }
 
 void
-print_spaces (int n, struct ui_file *file)
+print_spaces (register int n, register struct ui_file *file)
 {
   fputs_unfiltered (n_spaces (n), file);
 }
@@ -1277,8 +1295,8 @@ int
 query (const char *ctlstr, ...)
 {
   va_list args;
-  int answer;
-  int ans2;
+  register int answer;
+  register int ans2;
   int retval;
 
   va_start (args, ctlstr);
@@ -1346,145 +1364,6 @@ query (const char *ctlstr, ...)
 }
 
 
-/* This function supports the nquery() and yquery() functions.
-   Ask user a y-or-n question and return 0 if answer is no, 1 if
-   answer is yes, or default the answer to the specified default.
-   DEFCHAR is either 'y' or 'n' and refers to the default answer.
-   CTLSTR is the control string and should end in "? ".  It should
-   not say how to answer, because we do that.
-   ARGS are the arguments passed along with the CTLSTR argument to
-   printf.  */
-
-static int
-defaulted_query (const char *ctlstr, const char defchar, va_list args)
-{
-  int answer;
-  int ans2;
-  int retval;
-  int def_value;
-  char def_answer, not_def_answer;
-  char *y_string, *n_string;
-
-  /* Set up according to which answer is the default.  */
-  if (defchar == 'y')
-    {
-      def_value = 1;
-      def_answer = 'Y';
-      not_def_answer = 'N';
-      y_string = "[y]";
-      n_string = "n";
-    }
-  else
-    {
-      def_value = 0;
-      def_answer = 'N';
-      not_def_answer = 'Y';
-      y_string = "y";
-      n_string = "[n]";
-    }
-
-  if (query_hook)
-    {
-      return query_hook (ctlstr, args);
-    }
-
-  /* Automatically answer default value if input is not from a terminal.  */
-  if (!input_from_terminal_p ())
-    return def_value;
-
-  while (1)
-    {
-      wrap_here ("");		/* Flush any buffered output */
-      gdb_flush (gdb_stdout);
-
-      if (annotation_level > 1)
-	printf_filtered ("\n\032\032pre-%cquery\n", defchar);
-
-      vfprintf_filtered (gdb_stdout, ctlstr, args);
-      printf_filtered ("(%s or %s) ", y_string, n_string);
-
-      if (annotation_level > 1)
-	printf_filtered ("\n\032\032%cquery\n", defchar);
-
-      wrap_here ("");
-      gdb_flush (gdb_stdout);
-
-      answer = fgetc (stdin);
-      clearerr (stdin);		/* in case of C-d */
-      if (answer == EOF)	/* C-d */
-	{
-	  retval = def_value;
-	  break;
-	}
-      /* Eat rest of input line, to EOF or newline */
-      if (answer != '\n')
-	do
-	  {
-	    ans2 = fgetc (stdin);
-	    clearerr (stdin);
-	  }
-	while (ans2 != EOF && ans2 != '\n' && ans2 != '\r');
-
-      if (answer >= 'a')
-	answer -= 040;
-      /* Check answer.  For the non-default, the user must specify
-         the non-default explicitly.  */
-      if (answer == not_def_answer)
-	{
-	  retval = !def_value;
-	  break;
-	}
-      /* Otherwise, for the default, the user may either specify
-         the required input or have it default by entering nothing.  */
-      if (answer == def_answer || answer == '\n' || 
-	  answer == '\r' || answer == EOF)
-	{
-	  retval = def_value;
-	  break;
-	}
-      /* Invalid entries are not defaulted and require another selection.  */
-      printf_filtered ("Please answer %s or %s.\n",
-		       y_string, n_string);
-    }
-
-  if (annotation_level > 1)
-    printf_filtered ("\n\032\032post-%cquery\n", defchar);
-  return retval;
-}
-
-
-/* Ask user a y-or-n question and return 0 if answer is no, 1 if
-   answer is yes, or 0 if answer is defaulted.
-   Takes three args which are given to printf to print the question.
-   The first, a control string, should end in "? ".
-   It should not say how to answer, because we do that.  */
-
-int
-nquery (const char *ctlstr, ...)
-{
-  va_list args;
-
-  va_start (args, ctlstr);
-  return defaulted_query (ctlstr, 'n', args);
-  va_end (args);
-}
-
-/* Ask user a y-or-n question and return 0 if answer is no, 1 if
-   answer is yes, or 1 if answer is defaulted.
-   Takes three args which are given to printf to print the question.
-   The first, a control string, should end in "? ".
-   It should not say how to answer, because we do that.  */
-
-int
-yquery (const char *ctlstr, ...)
-{
-  va_list args;
-
-  va_start (args, ctlstr);
-  return defaulted_query (ctlstr, 'y', args);
-  va_end (args);
-}
-
 /* Print an error message saying that we couldn't make sense of a
    \^mumble sequence in a string or character constant.  START and END
    indicate a substring of some larger string that contains the
@@ -1521,7 +1400,7 @@ int
 parse_escape (char **string_ptr)
 {
   int target_char;
-  int c = *(*string_ptr)++;
+  register int c = *(*string_ptr)++;
   if (c_parse_backslash (c, &target_char))
     return target_char;
   else
@@ -1579,8 +1458,8 @@ parse_escape (char **string_ptr)
       case '6':
       case '7':
 	{
-	  int i = c - '0';
-	  int count = 0;
+	  register int i = c - '0';
+	  register int count = 0;
 	  while (++count < 3)
 	    {
 	      c = (**string_ptr);
@@ -1689,12 +1568,11 @@ fputstrn_unfiltered (const char *str, int n, int quoter,
 }
 
 
+
 /* Number of lines per page or UINT_MAX if paging is disabled.  */
 static unsigned int lines_per_page;
-
 /* Number of chars per line or UINT_MAX if line folding is disabled.  */
 static unsigned int chars_per_line;
-
 /* Current count of lines printed on this page, chars on this line.  */
 static unsigned int lines_printed, chars_printed;
 
@@ -1723,8 +1601,7 @@ static char *wrap_indent;
 static int wrap_column;
 
 
-/* Inialize the number of lines per page and chars per line.  */
-
+/* Inialize the lines and chars per page */
 void
 init_page_info (void)
 {
@@ -1732,67 +1609,66 @@ init_page_info (void)
   if (!tui_get_command_dimension (&chars_per_line, &lines_per_page))
 #endif
     {
-      int rows, cols;
-
+      /* These defaults will be used if we are unable to get the correct
+         values from termcap.  */
 #if defined(__GO32__)
-      rows = ScreenRows ();
-      cols = ScreenCols ();
-      lines_per_page = rows;
-      chars_per_line = cols;
+      lines_per_page = ScreenRows ();
+      chars_per_line = ScreenCols ();
 #else
-      /* Make sure Readline has initialized its terminal settings.  */
-      rl_reset_terminal (NULL);
+      lines_per_page = 24;
+      chars_per_line = 80;
 
-      /* Get the screen size from Readline.  */
-      rl_get_screen_size (&rows, &cols);
-      lines_per_page = rows;
-      chars_per_line = cols;
+#if !defined (_WIN32)
+      /* Initialize the screen height and width from termcap.  */
+      {
+	char *termtype = getenv ("TERM");
 
-      /* Readline should have fetched the termcap entry for us.  */
-      if (tgetnum ("li") < 0 || getenv ("EMACS"))
-	{
-	  /* The number of lines per page is not mentioned in the
-	     terminal description.  This probably means that paging is
-	     not useful (e.g. emacs shell window), so disable paging.  */
-	  lines_per_page = UINT_MAX;
-	}
+	/* Positive means success, nonpositive means failure.  */
+	int status;
 
-      /* FIXME: Get rid of this junk.  */
-#if defined(SIGWINCH) && defined(SIGWINCH_HANDLER)
-      SIGWINCH_HANDLER (SIGWINCH);
+	/* 2048 is large enough for all known terminals, according to the
+	   GNU termcap manual.  */
+	char term_buffer[2048];
+
+	if (termtype)
+	  {
+	    status = tgetent (term_buffer, termtype);
+	    if (status > 0)
+	      {
+		int val;
+		int running_in_emacs = getenv ("EMACS") != NULL;
+
+		val = tgetnum ("li");
+		if (val >= 0 && !running_in_emacs)
+		  lines_per_page = val;
+		else
+		  /* The number of lines per page is not mentioned
+		     in the terminal description.  This probably means
+		     that paging is not useful (e.g. emacs shell window),
+		     so disable paging.  */
+		  lines_per_page = UINT_MAX;
+
+		val = tgetnum ("co");
+		if (val >= 0)
+		  chars_per_line = val;
+	      }
+	  }
+      }
 #endif
 
+#if defined(SIGWINCH) && defined(SIGWINCH_HANDLER)
+
+      /* If there is a better way to determine the window size, use it. */
+      SIGWINCH_HANDLER (SIGWINCH);
+#endif
+#endif
       /* If the output is not a terminal, don't paginate it.  */
       if (!ui_file_isatty (gdb_stdout))
 	lines_per_page = UINT_MAX;
-#endif
-    }
-
-  set_screen_size ();
+    }				/* the command_line_version */
   set_width ();
 }
 
-/* Set the screen size based on LINES_PER_PAGE and CHARS_PER_LINE.  */
-
-static void
-set_screen_size (void)
-{
-  int rows = lines_per_page;
-  int cols = chars_per_line;
-
-  if (rows <= 0)
-    rows = INT_MAX;
-
-  if (cols <= 0)
-    rl_get_screen_size (NULL, &cols);
-
-  /* Update Readline's idea of the terminal size.  */
-  rl_set_screen_size (rows, cols);
-}
-
-/* Reinitialize WRAP_BUFFER according to the current value of
-   CHARS_PER_LINE.  */
-
 static void
 set_width (void)
 {
@@ -1806,22 +1682,16 @@ set_width (void)
     }
   else
     wrap_buffer = (char *) xrealloc (wrap_buffer, chars_per_line + 2);
-  wrap_pointer = wrap_buffer;	/* Start it at the beginning.  */
+  wrap_pointer = wrap_buffer;	/* Start it at the beginning */
 }
 
+/* ARGSUSED */
 static void
 set_width_command (char *args, int from_tty, struct cmd_list_element *c)
 {
-  set_screen_size ();
   set_width ();
 }
 
-static void
-set_height_command (char *args, int from_tty, struct cmd_list_element *c)
-{
-  set_screen_size ();
-}
-
 /* Wait, so the user can read what's on the screen.  Prompt the user
    to continue by pressing RETURN.  */
 
@@ -2617,19 +2487,27 @@ initialize_utils (void)
 {
   struct cmd_list_element *c;
 
-  c = add_set_cmd ("width", class_support, var_uinteger, &chars_per_line,
+  c = add_set_cmd ("width", class_support, var_uinteger,
+		   (char *) &chars_per_line,
 		   "Set number of characters gdb thinks are in a line.",
 		   &setlist);
   add_show_from_set (c, &showlist);
   set_cmd_sfunc (c, set_width_command);
 
-  c = add_set_cmd ("height", class_support, var_uinteger, &lines_per_page,
-		   "Set number of lines gdb thinks are in a page.", &setlist);
-  add_show_from_set (c, &showlist);
-  set_cmd_sfunc (c, set_height_command);
+  add_show_from_set
+    (add_set_cmd ("height", class_support,
+		  var_uinteger, (char *) &lines_per_page,
+		  "Set number of lines gdb thinks are in a page.", &setlist),
+     &showlist);
 
   init_page_info ();
 
+  /* If the output is not a terminal, don't paginate it.  */
+  if (!ui_file_isatty (gdb_stdout))
+    lines_per_page = UINT_MAX;
+
+  set_width_command ((char *) NULL, 0, c);
+
   add_show_from_set
     (add_set_cmd ("demangle", class_support, var_boolean,
 		  (char *) &demangle,
@@ -3052,19 +2930,3 @@ gnu_debuglink_crc32 (unsigned long crc, unsigned char *buf, size_t len)
     crc = crc32_table[(crc ^ *buf) & 0xff] ^ (crc >> 8);
   return ~crc & 0xffffffff;;
 }
-
-ULONGEST
-align_up (ULONGEST v, int n)
-{
-  /* Check that N is really a power of two.  */
-  gdb_assert (n && (n & (n-1)) == 0);
-  return (v + n - 1) & -n;
-}
-
-ULONGEST
-align_down (ULONGEST v, int n)
-{
-  /* Check that N is really a power of two.  */
-  gdb_assert (n && (n & (n-1)) == 0);
-  return (v & -n);
-}
