@@ -1,5 +1,7 @@
 /* Output generating routines for GDB.
-   Copyright 1999, 2000, 2001 Free Software Foundation, Inc.
+
+   Copyright 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+
    Contributed by Cygnus Solutions.
    Written by Fernando Nasser for Cygnus.
 
@@ -27,11 +29,6 @@
 #include "ui-out.h"
 #include "gdb_assert.h"
 
-/* Convenience macro for allocting typesafe memory. */
-
-#undef XMALLOC
-#define XMALLOC(TYPE) (TYPE*) xmalloc (sizeof (TYPE))
-
 /* table header structures */
 
 struct ui_out_hdr
@@ -48,7 +45,7 @@ struct ui_out_hdr
    is always available.  Stack/nested level 0 is reserved for the
    top-level result. */
 
-enum { MAX_UI_OUT_LEVELS = 5 };
+enum { MAX_UI_OUT_LEVELS = 6 };
 
 struct ui_out_level
   {
@@ -58,6 +55,43 @@ struct ui_out_level
     enum ui_out_type type;
   };
 
+/* Tables are special.  Maintain a separate structure that tracks
+   their state.  At present an output can only contain a single table
+   but that restriction might eventually be lifted.  */
+
+struct ui_out_table
+{
+  /* If on, a table is being generated.  */
+  int flag;
+
+  /* If on, the body of a table is being generated.  If off, the table
+     header is being generated.  */
+  int body_flag;
+
+  /* The level at which each entry of the table is to be found.  A row
+     (a tuple) is made up of entries.  Consequently ENTRY_LEVEL is one
+     above that of the table.  */
+  int entry_level;
+
+  /* Number of table columns (as specified in the table_begin call).  */
+  int columns;
+
+  /* String identifying the table (as specified in the table_begin
+     call).  */
+  char *id;
+
+  /* Points to the first table header (if any).  */
+  struct ui_out_hdr *header_first;
+
+  /* Points to the last table header (if any).  */
+  struct ui_out_hdr *header_last;
+
+  /* Points to header of NEXT column to format.  */
+  struct ui_out_hdr *header_next;
+
+};
+
+
 /* The ui_out structure */
 /* Any change here requires a corresponding one in the initialization
    of the default uiout, which is statically initialized */
@@ -69,31 +103,12 @@ struct ui_out
     struct ui_out_impl *impl;
     struct ui_out_data *data;
 
-    /* if on, a table is being generated */
-    int table_flag;
-
-    /* if on, the body of a table is being generated */
-    int body_flag;
-
-    /* number of table columns (as specified in the table_begin call) */
-    int table_columns;
-
-    /* strinf identifying the table (as specified in the table_begin call) */
-    char *table_id;
-
-    /* Sub structure tracking the table depth. */
+    /* Sub structure tracking the ui-out depth.  */
     int level;
     struct ui_out_level levels[MAX_UI_OUT_LEVELS];
 
-    /* points to the first header (if any) */
-    struct ui_out_hdr *headerfirst;
-
-    /* points to the last header (if any) */
-    struct ui_out_hdr *headerlast;
-
-    /* points to header of next column to format */
-    struct ui_out_hdr *headercurr;
-
+    /* A table, if any.  At present only a single table is supported.  */
+    struct ui_out_table table;
   };
 
 /* The current (inner most) level. */
@@ -191,6 +206,7 @@ struct ui_out_impl default_ui_out_impl =
   default_message,
   default_wrap_hint,
   default_flush,
+  NULL,
   0, /* Does not need MI hacks.  */
 };
 
@@ -239,6 +255,7 @@ static void uo_message (struct ui_out *uiout, int verbosity,
 			const char *format, va_list args);
 static void uo_wrap_hint (struct ui_out *uiout, char *identstring);
 static void uo_flush (struct ui_out *uiout);
+static int uo_redirect (struct ui_out *uiout, struct ui_file *outstream);
 
 /* Prototypes for local functions */
 
@@ -246,11 +263,11 @@ extern void _initialize_ui_out (void);
 static void append_header_to_list (struct ui_out *uiout, int width,
 				   int alignment, const char *col_name,
 				   const char *colhdr);
-static int get_curr_header (struct ui_out *uiout, int *colno, int *width,
+static int get_next_header (struct ui_out *uiout, int *colno, int *width,
 			    int *alignment, char **colhdr);
 static void clear_header_list (struct ui_out *uiout);
-static void verify_field_proper_position (struct ui_out *uiout);
-static void verify_field_alignment (struct ui_out *uiout, int fldno, int *width, int *alignment);
+static void verify_field (struct ui_out *uiout, int *fldno, int *width,
+			  int *align);
 
 static void init_ui_out_state (struct ui_out *uiout);
 
@@ -258,63 +275,66 @@ static void init_ui_out_state (struct ui_out *uiout);
 
 /* Mark beginning of a table */
 
-void
+static void
 ui_out_table_begin (struct ui_out *uiout, int nbrofcols,
 		    int nr_rows,
 		    const char *tblid)
 {
-  if (uiout->table_flag)
+  if (uiout->table.flag)
     internal_error (__FILE__, __LINE__,
 		    "tables cannot be nested; table_begin found before \
 previous table_end.");
 
-  uiout->table_flag = 1;
-  uiout->table_columns = nbrofcols;
+  uiout->table.flag = 1;
+  uiout->table.body_flag = 0;
+  uiout->table.entry_level = uiout->level + 1;
+  uiout->table.columns = nbrofcols;
   if (tblid != NULL)
-    uiout->table_id = xstrdup (tblid);
+    uiout->table.id = xstrdup (tblid);
   else
-    uiout->table_id = NULL;
+    uiout->table.id = NULL;
   clear_header_list (uiout);
 
-  uo_table_begin (uiout, nbrofcols, nr_rows, uiout->table_id);
+  uo_table_begin (uiout, nbrofcols, nr_rows, uiout->table.id);
 }
 
 void
 ui_out_table_body (struct ui_out *uiout)
 {
-  if (!uiout->table_flag)
+  if (!uiout->table.flag)
     internal_error (__FILE__, __LINE__,
 		    "table_body outside a table is not valid; it must be \
 after a table_begin and before a table_end.");
-  if (uiout->body_flag)
+  if (uiout->table.body_flag)
     internal_error (__FILE__, __LINE__,
 		    "extra table_body call not allowed; there must be \
 only one table_body after a table_begin and before a table_end.");
-  if (uiout->headercurr->colno != uiout->table_columns)
+  if (uiout->table.header_next->colno != uiout->table.columns)
     internal_error (__FILE__, __LINE__,
 		    "number of headers differ from number of table \
 columns.");
 
-  uiout->body_flag = 1;
-  uiout->headercurr = uiout->headerfirst;
+  uiout->table.body_flag = 1;
+  uiout->table.header_next = uiout->table.header_first;
 
   uo_table_body (uiout);
 }
 
-void
+static void
 ui_out_table_end (struct ui_out *uiout)
 {
-  if (!uiout->table_flag)
+  if (!uiout->table.flag)
     internal_error (__FILE__, __LINE__,
 		    "misplaced table_end or missing table_begin.");
 
-  uiout->body_flag = 0;
-  uiout->table_flag = 0;
+  uiout->table.entry_level = 0;
+  uiout->table.body_flag = 0;
+  uiout->table.flag = 0;
 
   uo_table_end (uiout);
 
-  if (uiout->table_id)
-    xfree (uiout->table_id);
+  if (uiout->table.id)
+    xfree (uiout->table.id);
   clear_header_list (uiout);
 }
 
@@ -323,7 +343,7 @@ ui_out_table_header (struct ui_out *uiout, int width, enum ui_align alignment,
 		     const char *col_name,
 		     const char *colhdr)
 {
-  if (!uiout->table_flag || uiout->body_flag)
+  if (!uiout->table.flag || uiout->table.body_flag)
     internal_error (__FILE__, __LINE__,
 		    "table header must be specified after table_begin \
 and before table_body.");
@@ -333,35 +353,58 @@ and before table_body.");
   uo_table_header (uiout, width, alignment, col_name, colhdr);
 }
 
+static void
+do_cleanup_table_end (void *data)
+{
+  struct ui_out *ui_out = data;
+
+  ui_out_table_end (ui_out);
+}
+
+struct cleanup *
+make_cleanup_ui_out_table_begin_end (struct ui_out *ui_out, int nr_cols,
+                                     int nr_rows, const char *tblid)
+{
+  ui_out_table_begin (ui_out, nr_cols, nr_rows, tblid);
+  return make_cleanup (do_cleanup_table_end, ui_out);
+}
+
 void
 ui_out_begin (struct ui_out *uiout,
 	      enum ui_out_type type,
 	      const char *id)
 {
   int new_level;
-  if (uiout->table_flag && !uiout->body_flag)
+  if (uiout->table.flag && !uiout->table.body_flag)
     internal_error (__FILE__, __LINE__,
 		    "table header or table_body expected; lists must be \
 specified after table_body.");
+
+  /* Be careful to verify the ``field'' before the new tuple/list is
+     pushed onto the stack.  That way the containing list/table/row is
+     verified and not the newly created tuple/list.  This verification
+     is needed (at least) for the case where a table row entry
+     contains either a tuple/list.  For that case bookkeeping such as
+     updating the column count or advancing to the next heading still
+     needs to be performed.  */
+  {
+    int fldno;
+    int width;
+    int align;
+    verify_field (uiout, &fldno, &width, &align);
+  }
+
   new_level = push_level (uiout, type, id);
-  if (uiout->table_flag && (new_level == 1))
-    uiout->headercurr = uiout->headerfirst;
+
+  /* If the push puts us at the same level as a table row entry, we've
+     got a new table row.  Put the header pointer back to the start.  */
+  if (uiout->table.body_flag
+      && uiout->table.entry_level == new_level)
+    uiout->table.header_next = uiout->table.header_first;
+
   uo_begin (uiout, type, new_level, id);
 }
 
-void
-ui_out_list_begin (struct ui_out *uiout,
-		   const char *id)
-{
-  ui_out_begin (uiout, ui_out_type_list, id);
-}
-
-void
-ui_out_tuple_begin (struct ui_out *uiout, const char *id)
-{
-  ui_out_begin (uiout, ui_out_type_tuple, id);
-}
-
 void
 ui_out_end (struct ui_out *uiout,
 	    enum ui_out_type type)
@@ -370,18 +413,6 @@ ui_out_end (struct ui_out *uiout,
   uo_end (uiout, type, old_level);
 }
 
-void
-ui_out_list_end (struct ui_out *uiout)
-{
-  ui_out_end (uiout, ui_out_type_list);
-}
-
-void
-ui_out_tuple_end (struct ui_out *uiout)
-{
-  ui_out_end (uiout, ui_out_type_tuple);
-}
-
 struct ui_out_end_cleanup_data
 {
   struct ui_out *uiout;
@@ -407,20 +438,11 @@ make_cleanup_ui_out_end (struct ui_out *uiout,
   return make_cleanup (do_cleanup_end, end_cleanup_data);
 }
 
-struct cleanup *
-make_cleanup_ui_out_begin_end (struct ui_out *uiout,
-			       enum ui_out_type type,
-			       const char *id)
-{
-  ui_out_begin (uiout, type, id);
-  return make_cleanup_ui_out_end (uiout, type);
-}
-
 struct cleanup *
 make_cleanup_ui_out_tuple_begin_end (struct ui_out *uiout,
 				     const char *id)
 {
-  ui_out_tuple_begin (uiout, id);
+  ui_out_begin (uiout, ui_out_type_tuple, id);
   return make_cleanup_ui_out_end (uiout, ui_out_type_tuple);
 }
 
@@ -428,7 +450,7 @@ struct cleanup *
 make_cleanup_ui_out_list_begin_end (struct ui_out *uiout,
 				    const char *id)
 {
-  ui_out_list_begin (uiout, id);
+  ui_out_begin (uiout, ui_out_type_list, id);
   return make_cleanup_ui_out_end (uiout, ui_out_type_list);
 }
 
@@ -442,16 +464,28 @@ ui_out_field_int (struct ui_out *uiout,
   int align;
   struct ui_out_level *current = current_level (uiout);
 
-  verify_field_proper_position (uiout);
-
-  current->field_count += 1;
-  fldno = current->field_count;
-
-  verify_field_alignment (uiout, fldno, &width, &align);
+  verify_field (uiout, &fldno, &width, &align);
 
   uo_field_int (uiout, fldno, width, align, fldname, value);
 }
 
+void
+ui_out_field_fmt_int (struct ui_out *uiout,
+                      int input_width,
+                      enum ui_align input_align,
+		      const char *fldname,
+		      int value)
+{
+  int fldno;
+  int width;
+  int align;
+  struct ui_out_level *current = current_level (uiout);
+
+  verify_field (uiout, &fldno, &width, &align);
+
+  uo_field_int (uiout, fldno, input_width, input_align, fldname, value);
+}
+
 void
 ui_out_field_core_addr (struct ui_out *uiout,
 			const char *fldname,
@@ -459,9 +493,14 @@ ui_out_field_core_addr (struct ui_out *uiout,
 {
   char addstr[20];
 
-  /* FIXME-32x64: need a print_address_numeric with field width */
+  /* FIXME: cagney/2002-05-03: Need local_address_string() function
+     that returns the language localized string formatted to a width
+     based on TARGET_ADDR_BIT.  */
   /* print_address_numeric (address, 1, local_stream); */
-  strcpy (addstr, local_hex_string_custom ((unsigned long) address, "08l"));
+  if (TARGET_ADDR_BIT <= 32)
+    strcpy (addstr, local_hex_string_custom (address, "08l"));
+  else
+    strcpy (addstr, local_hex_string_custom (address, "016l"));
 
   ui_out_field_string (uiout, fldname, addstr);
 }
@@ -491,14 +530,8 @@ ui_out_field_skip (struct ui_out *uiout,
   int fldno;
   int width;
   int align;
-  struct ui_out_level *current = current_level (uiout);
 
-  verify_field_proper_position (uiout);
-
-  current->field_count += 1;
-  fldno = current->field_count;
-
-  verify_field_alignment (uiout, fldno, &width, &align);
+  verify_field (uiout, &fldno, &width, &align);
 
   uo_field_skip (uiout, fldno, width, align, fldname);
 }
@@ -511,14 +544,8 @@ ui_out_field_string (struct ui_out *uiout,
   int fldno;
   int width;
   int align;
-  struct ui_out_level *current = current_level (uiout);
 
-  verify_field_proper_position (uiout);
-
-  current->field_count += 1;
-  fldno = current->field_count;
-
-  verify_field_alignment (uiout, fldno, &width, &align);
+  verify_field (uiout, &fldno, &width, &align);
 
   uo_field_string (uiout, fldno, width, align, fldname, string);
 }
@@ -533,15 +560,9 @@ ui_out_field_fmt (struct ui_out *uiout,
   int fldno;
   int width;
   int align;
-  struct ui_out_level *current = current_level (uiout);
-
-  verify_field_proper_position (uiout);
-
-  current->field_count += 1;
-  fldno = current->field_count;
 
   /* will not align, but has to call anyway */
-  verify_field_alignment (uiout, fldno, &width, &align);
+  verify_field (uiout, &fldno, &width, &align);
 
   va_start (args, format);
 
@@ -619,6 +640,12 @@ ui_out_flush (struct ui_out *uiout)
   uo_flush (uiout);
 }
 
+int
+ui_out_redirect (struct ui_out *uiout, struct ui_file *outstream)
+{
+  return uo_redirect (uiout, outstream);
+}
+
 /* set the flags specified by the mask given */
 int
 ui_out_set_flags (struct ui_out *uiout, int mask)
@@ -962,6 +989,15 @@ uo_flush (struct ui_out *uiout)
   uiout->impl->flush (uiout);
 }
 
+int
+uo_redirect (struct ui_out *uiout, struct ui_file *outstream)
+{
+  if (!uiout->impl->redirect)
+    return -1;
+  uiout->impl->redirect (uiout, outstream);
+  return 0;
+}
+
 /* local functions */
 
 /* list of column headers manipulation routines */
@@ -969,16 +1005,17 @@ uo_flush (struct ui_out *uiout)
 static void
 clear_header_list (struct ui_out *uiout)
 {
-  while (uiout->headerfirst != NULL)
+  while (uiout->table.header_first != NULL)
     {
-      uiout->headercurr = uiout->headerfirst;
-      uiout->headerfirst = uiout->headerfirst->next;
-      if (uiout->headercurr->colhdr != NULL)
-	xfree (uiout->headercurr->colhdr);
-      xfree (uiout->headercurr);
+      uiout->table.header_next = uiout->table.header_first;
+      uiout->table.header_first = uiout->table.header_first->next;
+      if (uiout->table.header_next->colhdr != NULL)
+	xfree (uiout->table.header_next->colhdr);
+      xfree (uiout->table.header_next);
     }
-  uiout->headerlast = NULL;
-  uiout->headercurr = NULL;
+  gdb_assert (uiout->table.header_first == NULL);
+  uiout->table.header_last = NULL;
+  uiout->table.header_next = NULL;
 }
 
 static void
@@ -1003,74 +1040,74 @@ append_header_to_list (struct ui_out *uiout,
   else
     temphdr->col_name = xstrdup (colhdr);
   temphdr->next = NULL;
-  if (uiout->headerfirst == NULL)
+  if (uiout->table.header_first == NULL)
     {
       temphdr->colno = 1;
-      uiout->headerfirst = temphdr;
-      uiout->headerlast = temphdr;
+      uiout->table.header_first = temphdr;
+      uiout->table.header_last = temphdr;
     }
   else
     {
-      temphdr->colno = uiout->headerlast->colno + 1;
-      uiout->headerlast->next = temphdr;
-      uiout->headerlast = temphdr;
+      temphdr->colno = uiout->table.header_last->colno + 1;
+      uiout->table.header_last->next = temphdr;
+      uiout->table.header_last = temphdr;
     }
-  uiout->headercurr = uiout->headerlast;
+  uiout->table.header_next = uiout->table.header_last;
 }
 
-/* returns 0 if there is no more headers */
+/* Extract the format information for the NEXT header and and advance
+   the header pointer.  Return 0 if there was no next header.  */
 
 static int
-get_curr_header (struct ui_out *uiout,
+get_next_header (struct ui_out *uiout,
 		 int *colno,
 		 int *width,
 		 int *alignment,
 		 char **colhdr)
 {
-  /* There may be no headers at all or we may have used all columns */
-  if (uiout->headercurr == NULL)
+  /* There may be no headers at all or we may have used all columns.  */
+  if (uiout->table.header_next == NULL)
     return 0;
-  *colno = uiout->headercurr->colno;
-  *width = uiout->headercurr->width;
-  *alignment = uiout->headercurr->alignment;
-  *colhdr = uiout->headercurr->colhdr;
-  uiout->headercurr = uiout->headercurr->next;
+  *colno = uiout->table.header_next->colno;
+  *width = uiout->table.header_next->width;
+  *alignment = uiout->table.header_next->alignment;
+  *colhdr = uiout->table.header_next->colhdr;
+  /* Advance the header pointer to the next entry.  */
+  uiout->table.header_next = uiout->table.header_next->next;
   return 1;
 }
 
-/* makes sure the field_* calls were properly placed */
+
+/* Verify that the field/tuple/list is correctly positioned.  Return
+   the field number and corresponding alignment (if
+   available/applicable).  */
 
 static void
-verify_field_proper_position (struct ui_out *uiout)
+verify_field (struct ui_out *uiout, int *fldno, int *width, int *align)
 {
-  if (uiout->table_flag)
+  struct ui_out_level *current = current_level (uiout);
+  char *text;
+
+  if (uiout->table.flag)
     {
-      if (!uiout->body_flag)
+      if (!uiout->table.body_flag)
 	internal_error (__FILE__, __LINE__,
 			"table_body missing; table fields must be \
 specified after table_body and inside a list.");
-      if (uiout->level == 0)
-	internal_error (__FILE__, __LINE__,
-			"list_begin missing; table fields must be \
-specified after table_body and inside a list.");
+      /* NOTE: cagney/2001-12-08: There was a check here to ensure
+	 that this code was only executed when uiout->level was
+	 greater than zero.  That no longer applies - this code is run
+	 before each table row tuple is started and at that point the
+	 level is zero.  */
     }
-}
 
-/* determines what is the alignment policy */
+  current->field_count += 1;
 
-static void
-verify_field_alignment (struct ui_out *uiout,
-			int fldno,
-			int *width,
-			int *align)
-{
-  int colno;
-  char *text;
-
-  if (uiout->table_flag
-      && get_curr_header (uiout, &colno, width, align, &text))
+  if (uiout->table.body_flag
+      && uiout->table.entry_level == uiout->level
+      && get_next_header (uiout, fldno, width, align, &text))
     {
-      if (fldno != colno)
+      if (*fldno != current->field_count)
 	internal_error (__FILE__, __LINE__,
 			"ui-out internal error in handling headers.");
     }
@@ -1078,9 +1115,11 @@ verify_field_alignment (struct ui_out *uiout,
     {
       *width = 0;
       *align = ui_noalign;
+      *fldno = current->field_count;
     }
 }
 
+
 /* access to ui_out format private members */
 
 void
@@ -1107,13 +1146,13 @@ ui_out_new (struct ui_out_impl *impl,
   uiout->data = data;
   uiout->impl = impl;
   uiout->flags = flags;
-  uiout->table_flag = 0;
-  uiout->body_flag = 0;
+  uiout->table.flag = 0;
+  uiout->table.body_flag = 0;
   uiout->level = 0;
   memset (uiout->levels, 0, sizeof (uiout->levels));
-  uiout->headerfirst = NULL;
-  uiout->headerlast = NULL;
-  uiout->headercurr = NULL;
+  uiout->table.header_first = NULL;
+  uiout->table.header_last = NULL;
+  uiout->table.header_next = NULL;
   return uiout;
 }
 
