@@ -24,7 +24,7 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.25 $ $Date: 2004/05/06 20:03:09 $
+ * CVS: $Revision: 1.28 $ $Date: 2004/11/03 16:43:43 $
  */
 
 #include "defs.h"
@@ -43,13 +43,27 @@ gdb_main_loop(int argc, char **argv)
 {
 	argc = 1;
 
+	if ((THIS_GCC_VERSION >= GCC(3,4,0)) && !(pc->flags & READNOW)) {
+		error(WARNING, 
+ "Because this kernel was compiled with gcc version %d.%d.%d, certain\n" 
+ "         commands or command options may fail unless crash is invoked with\n"
+ "         the  \"--readnow\" command line option.\n\n",
+			kt->gcc_version[0],
+			kt->gcc_version[1],
+			kt->gcc_version[2]);
+	}
+
 	if (pc->flags & SILENT) {
+		if (pc->flags & READNOW)
+			argv[argc++] = "--readnow";
 		argv[argc++] = "--quiet";
 		argv[argc++] = pc->namelist_debug ? 
 			pc->namelist_debug : 
 			(pc->debuginfo_file && (st->flags & CRC_MATCHES) ?
 			pc->debuginfo_file : pc->namelist);
 	} else {
+		if (pc->flags & READNOW)
+			argv[argc++] = "--readnow";
 		argv[argc++] = pc->namelist_debug ? 
 			pc->namelist_debug : 
 			(pc->debuginfo_file && (st->flags & CRC_MATCHES) ?
@@ -67,14 +81,14 @@ gdb_main_loop(int argc, char **argv)
         optind = 0;
         command_loop_hook = main_loop;
 
-#if defined(GDB_5_3) || defined(GDB_6_1)
+#if defined(GDB_5_3) || defined(GDB_6_0) || defined(GDB_6_1)
         gdb_main_entry(argc, argv);
 #else
         gdb_main(argc, argv);
 #endif
 }
 
-#if defined(GDB_6_1)
+#if defined(GDB_6_0) || defined(GDB_6_1)
 /*
  *  Update any hooks that gdb has set.
  */
@@ -98,7 +112,7 @@ display_gdb_banner(void)
 	optind = 0;
         command_loop_hook = exit_after_gdb_info;
 	args[0] = "gdb";
-#if defined(GDB_5_3) || defined(GDB_6_1)
+#if defined(GDB_5_3) || defined(GDB_6_0) || defined(GDB_6_1)
         gdb_main_entry(1, args);
 #else
         gdb_main(1, args);
@@ -132,6 +146,7 @@ void
 gdb_session_init(void)
 {
 	struct gnu_request *req;
+	int debug_data_pulled_in;
 
         if (!have_partial_symbols() && !have_full_symbols())
 		no_debugging_data(FATAL);
@@ -170,7 +185,7 @@ gdb_session_init(void)
 	prettyprint_structs = 1;
 	repeat_count_threshold = 0x7fffffff;
 	print_max = 256;
-#if !defined(GDB_6_1)
+#if !defined(GDB_6_0) && !defined(GDB_6_1)
 	gdb_disassemble_from_exec = 0;
 #endif
 
@@ -180,14 +195,34 @@ gdb_session_init(void)
         req->buf = GETBUF(BUFSIZE);
 
 	/*
-	 *  Make sure the namelist has symbolic data...
-	 */
+	 *  Make sure the namelist has symbolic data.  Later versions of
+	 *  gcc may require that debug data be pulled in by printing a 
+	 *  static kernel data structure.
+  	 */
+	debug_data_pulled_in = FALSE;
+retry:
+	BZERO(req->buf, BUFSIZE);
         req->command = GNU_GET_DATATYPE;
         req->name = "task_struct";
-        req->flags |= GNU_RETURN_ON_ERROR;
+        req->flags = GNU_RETURN_ON_ERROR;
         gdb_interface(req);
-        if (req->flags & GNU_COMMAND_FAILED)
+
+        if (req->flags & GNU_COMMAND_FAILED) {
+		if (!debug_data_pulled_in) {
+			if (CRASHDEBUG(1))
+				error(INFO, 
+         "gdb_session_init: pulling in debug data by accessing init_mm.mmap\n");
+			debug_data_pulled_in = TRUE;
+			req->command = GNU_PASS_THROUGH;
+			req->flags = GNU_RETURN_ON_ERROR|GNU_NO_READMEM;
+			req->name = NULL;
+			sprintf(req->buf, "print init_mm.mmap");
+			gdb_interface(req);
+        		if (!(req->flags & GNU_COMMAND_FAILED)) 
+				goto retry;
+		}
 		no_debugging_data(WARNING);
+	}
 
 	if (pc->flags & KERNEL_DEBUG_QUERY) {
 		fprintf(fp, "\n%s: %s: contains debugging data\n\n",
@@ -398,6 +433,8 @@ dump_gnu_request(struct gnu_request *req, int in_gdb)
                 console("%sGNU_RETURN_ON_ERROR", others++ ? "|" : "");
         if (req->flags & GNU_FROM_TTY_OFF)
                 console("%sGNU_FROM_TTY_OFF", others++ ? "|" : "");
+        if (req->flags & GNU_NO_READMEM)
+                console("%sGNU_NO_READMEM", others++ ? "|" : "");
 	console(")\n");
 
         console("addr: %lx ", req->addr);
@@ -687,6 +724,9 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 	if (write)
 		return FALSE;
 
+	if (pc->cur_req->flags & GNU_NO_READMEM)
+		return TRUE;
+
 	if (UNIQUE_COMMAND("dis"))
 		memtype = UVADDR;
 	else if (!IS_KVADDR(addr)) {
