@@ -18,7 +18,7 @@
  *
  * 09/28/00  Transition to CVS version control
  *
- * CVS: $Revision: 1.58 $ $Date: 2004/06/14 16:57:43 $
+ * CVS: $Revision: 1.64 $ $Date: 2005/02/09 21:12:05 $
  */ 
 #ifdef IA64 
 #include "defs.h"
@@ -56,6 +56,7 @@ static void ia64_post_init(void);
 static struct line_number_hook ia64_line_number_hooks[];
 static ulong ia64_get_stackbase(ulong);
 static ulong ia64_get_stacktop(ulong);
+static void parse_cmdline_arg(void);
 
 struct unw_frame_info;
 static void dump_unw_frame_info(struct unw_frame_info *);
@@ -112,11 +113,11 @@ ia64_init(int when)
                 machdep->last_ptbl_read = 0;
 		machdep->verify_paddr = ia64_verify_paddr;
 		machdep->ptrs_per_pgd = PTRS_PER_PGD;
-                if (machdep->cmdline_arg)
-                        machdep->machspec->phys_start =
-                            htol(machdep->cmdline_arg, RETURN_ON_ERROR, NULL);
-                else
-                        machdep->machspec->phys_start = UNKNOWN_PHYS_START;
+                machdep->machspec->phys_start = UNKNOWN_PHYS_START;
+                if (machdep->cmdline_arg) 
+			parse_cmdline_arg();
+		if (ACTIVE())
+			machdep->flags |= DEVMEMRD;
                 break;     
 
         case PRE_GDB:
@@ -217,6 +218,108 @@ ia64_init(int when)
 	}
 }
 
+/*
+ *  --machdep <addr> defaults to the physical start location.
+ *
+ *  Otherwise, it's got to be a "item=value" string, separated
+ *  by commas if more than one is passed in.
+ */
+
+void
+parse_cmdline_arg(void)
+{
+	int i, c, errflag;
+	char *p;
+	char buf[BUFSIZE];
+	char *arglist[MAXARGS];
+	ulong value;
+        struct machine_specific *ms;
+
+        ms = &ia64_machine_specific;
+
+	if (!strstr(machdep->cmdline_arg, "=")) {
+		errflag = 0;
+        	value = htol(machdep->cmdline_arg,
+                	RETURN_ON_ERROR|QUIET, &errflag);
+		if (!errflag) {
+        		ms->phys_start = value;
+			error(NOTE, "setting phys_start to: 0x%lx\n",
+				ms->phys_start);
+		} else
+			error(WARNING, "ignoring --machdep option: %s\n\n",
+				machdep->cmdline_arg);
+		return;
+        }
+
+	strcpy(buf, machdep->cmdline_arg);
+
+	for (p = buf; *p; p++) {
+		if (*p == ',')
+			 *p = ' ';
+	}
+
+	c = parse_line(buf, arglist);
+
+	for (i = 0; i < c; i++) {
+		errflag = 0;
+
+		if (STRNEQ(arglist[i], "phys_start=")) {
+			p = arglist[i] + strlen("phys_start=");
+			if (strlen(p)) {
+        			value = htol(p, RETURN_ON_ERROR|QUIET, 
+					&errflag);
+				if (!errflag) {
+        				ms->phys_start = value;
+					error(NOTE, 
+					    "setting phys_start to: 0x%lx\n",
+						ms->phys_start);
+					continue;
+				}
+			}
+		} else if (STRNEQ(arglist[i], "init_stack_size=")) {
+			p = arglist[i] + strlen("init_stack_size=");
+			if (strlen(p)) {
+				value = stol(p, RETURN_ON_ERROR|QUIET, 
+					&errflag);
+				if (!errflag) {
+					ms->ia64_init_stack_size = (int)value;
+					error(NOTE, 
+		    	    	      "setting init_stack_size to: 0x%x (%d)\n",
+				    		ms->ia64_init_stack_size,
+				    		ms->ia64_init_stack_size);
+					continue;
+				}
+			}
+		}
+
+		error(WARNING, "ignoring --machdep option: %s\n", arglist[i]);
+	} 
+
+	if (c)
+		fprintf(fp, "\n");
+}
+
+
+int
+ia64_in_init_stack(ulong addr)
+{
+	ulong init_stack_addr;
+
+	if (!symbol_exists("ia64_init_stack"))
+		return FALSE;
+
+	/* 
+	 *  ia64_init_stack could be aliased to region 5 
+	 */
+	init_stack_addr = ia64_VTOP(symbol_value("ia64_init_stack"));
+	addr = ia64_VTOP(addr);
+	if ((addr < init_stack_addr) ||
+	    (addr >= (init_stack_addr+machdep->machspec->ia64_init_stack_size)))
+		return FALSE;
+
+	return TRUE;
+}
+
 void
 ia64_dump_machdep_table(ulong arg)
 {
@@ -300,10 +403,16 @@ ia64_dump_machdep_table(ulong arg)
 		fprintf(fp, "%sUNW_READ", others++ ? "|" : "");
 	if (machdep->flags & UNW_PTREGS)
 		fprintf(fp, "%sUNW_PTREGS", others++ ? "|" : "");
+	if (machdep->flags & UNW_R0)
+		fprintf(fp, "%sUNW_R0", others++ ? "|" : "");
 	if (machdep->flags & MEM_LIMIT)
 		fprintf(fp, "%sMEM_LIMIT", others++ ? "|" : "");
 	if (machdep->flags & SYSRQ)
 		fprintf(fp, "%sSYSRQ", others++ ? "|" : "");
+	if (machdep->flags & DEVMEMRD)
+		fprintf(fp, "%sDEVMEMRD", others++ ? "|" : "");
+	if (machdep->flags & INIT)
+		fprintf(fp, "%sINIT", others++ ? "|" : "");
         fprintf(fp, ")\n");
         fprintf(fp, "             kvbase: %lx\n", machdep->kvbase);
 	fprintf(fp, "  identity_map_base: %lx\n", machdep->identity_map_base);
@@ -441,6 +550,9 @@ ia64_dump_machdep_table(ulong arg)
         else
                 fprintf(fp, "%lx\n", (ulong)ms->unwind_debug);
 
+	fprintf(fp, "               ia64_init_stack_size: %d\n", 
+		ms->ia64_init_stack_size);
+
 	if (verbose)
 		ia64_display_memmap();
 }
@@ -866,7 +978,9 @@ ulong
 ia64_get_switch_stack(ulong task)
 {
 	ulong sw;
-
+		
+	if (LKCD_DUMPFILE() && (sw = get_lkcd_switch_stack(task)))
+	    return sw;
 	/*
 	 * debug only: get panic switch_stack from the ELF header.
 	 */
@@ -1070,9 +1184,9 @@ ia64_dis_filter(ulong vaddr, char *inbuf)
 {
         char buf1[BUFSIZE];
         char buf2[BUFSIZE];
-        char *colon, *p1;
+        char *colon, *p1, *p2;
         int argc;
-	int revise, stop_bit;
+	int revise_bracket, stop_bit;
         char *argv[MAXARGS];
         ulong value;
 
@@ -1098,25 +1212,24 @@ ia64_dis_filter(ulong vaddr, char *inbuf)
         strcpy(buf1, inbuf);
         argc = parse_line(buf1, argv);
 
-	revise = stop_bit = 0;
+	revise_bracket = stop_bit = 0;
 	if ((FIRSTCHAR(argv[argc-1]) == '<') &&
             (LASTCHAR(argv[argc-1]) == '>')) {
-		revise = TRUE;
+		revise_bracket = TRUE;
 		stop_bit = FALSE;
 	} else if ((FIRSTCHAR(argv[argc-1]) == '<') &&
             strstr(argv[argc-1], ">;;")) {
-		revise = TRUE;
+		revise_bracket = TRUE;
 		stop_bit = TRUE;
 	}
 
-        if (revise) {
+        if (revise_bracket) {
                 p1 = rindex(inbuf, '<');
-                while ((p1 > inbuf) && !STRNEQ(p1, " 0x"))
+                while ((p1 > inbuf) && !STRNEQ(p1, "0x"))
                         p1--;
 
-                if (!STRNEQ(p1, " 0x"))
+                if (!STRNEQ(p1, "0x"))
                         return FALSE;
-                p1++;
 
                 if (!extract_hex(p1, &value, NULLCHAR, TRUE))
                         return FALSE;
@@ -1126,11 +1239,47 @@ ia64_dis_filter(ulong vaddr, char *inbuf)
 			stop_bit ? ";;" : "");
 
                 sprintf(p1, "%s", buf1);
-        }
+
+        } else if (STRNEQ(argv[argc-2], "br.call.") &&
+		 STRNEQ(argv[argc-1], "b0=0x")) {
+		/*  
+		 *  Update module function calls of these formats:
+	 	 *
+		 *     br.call.sptk.many b0=0xa0000000003d5e40;;
+		 *     br.call.sptk.many b0=0xa00000000001dfc0
+		 *
+		 *  to show a bracketed function name if the destination
+		 *  address is a known symbol with no offset.
+		 */
+		if ((p1 = strstr(argv[argc-1], ";;")) &&
+		    (p2 = strstr(inbuf, ";;\n"))) {
+			*p1 = NULLCHAR;
+			p1 = &argv[argc-1][3];
+
+                	if (extract_hex(p1, &value, NULLCHAR, TRUE)) {
+				sprintf(buf1, " <%s>;;\n",
+					value_to_symstr(value, buf2, 
+					pc->output_radix));
+				if (IS_MODULE_VADDR(value) &&
+				    !strstr(buf2, "+"))
+					sprintf(p2, buf1);
+			} 
+		} else {
+			p1 = &argv[argc-1][3];
+			p2 = &LASTCHAR(inbuf);
+                	if (extract_hex(p1, &value, '\n', TRUE)) {
+				sprintf(buf1, " <%s>\n",
+					value_to_symstr(value, buf2, 
+					pc->output_radix));
+				if (IS_MODULE_VADDR(value) &&
+				    !strstr(buf2, "+"))
+					sprintf(p2, buf1);
+			}
+		}
+	}
 
         console("    %s", inbuf);
 
-
 	return TRUE;
 }
 
@@ -1150,16 +1299,21 @@ enum pt_reg_names {
 		P_f6_lo, P_f6_hi,
 		P_f7_lo, P_f7_hi,
 		P_f8_lo, P_f8_hi,
-		P_f9_lo, P_f9_hi, NUM_PT_REGS};
+		P_f9_lo, P_f9_hi, 
+		P_f10_lo, P_f10_hi, 
+		P_f11_lo, P_f11_hi, 
+		NUM_PT_REGS};
  
 void
 ia64_exception_frame(ulong addr, struct bt_info *bt)
 {
-	char buf[BUFSIZE], *p;
+	char buf[BUFSIZE], *p, *p1;
 	int fval;
 	ulong value1, value2;
 	ulong eframe[NUM_PT_REGS];
 
+	console("ia64_exception_frame: pt_regs: %lx\n", addr);
+
         if (bt->debug)
                 CRASHDEBUG_RESTORE();
 	CRASHDEBUG_SUSPEND(0);
@@ -1190,14 +1344,26 @@ ia64_exception_frame(ulong addr, struct bt_info *bt)
 			continue;
 		}
 
+                if (strstr(buf, "f10 = ")) {
+                        fval = 10;
+                        continue;
+                }
+
+                if (strstr(buf, "f11 = ")) {
+                        fval = 11;
+                        continue;
+                }
+
                 if (!strstr(buf, "0x"))
                         continue;
 
 		if (fval) {
 			p = strstr(buf, "0x");
+			if ((p1 = strstr(p, "}")))
+				*p1 = NULLCHAR;
 			extract_hex(p, &value1, ',', TRUE);
 			p = strstr(buf, ",");
-			extract_hex(p, &value2, '}', FALSE);  
+			extract_hex(p, &value2, NULLCHAR, FALSE);  
 			switch (fval)
 			{
 			case 6:
@@ -1216,6 +1382,14 @@ ia64_exception_frame(ulong addr, struct bt_info *bt)
 				eframe[P_f9_lo] = value1;
 				eframe[P_f9_hi] = value2;
 				break;
+                        case 10:
+                                eframe[P_f10_lo] = value1;
+                                eframe[P_f10_hi] = value2;
+                                break;
+                        case 11:
+                                eframe[P_f11_lo] = value1;
+                                eframe[P_f11_hi] = value2;
+                                break;
 			}
 			fval = 0;
 			continue;
@@ -1460,6 +1634,13 @@ ia64_exception_frame(ulong addr, struct bt_info *bt)
         fprintf(fp, "   F9: %05lx%016lx\n",
                 eframe[P_f9_hi], eframe[P_f9_lo]);
 
+	if (machdep->flags & NEW_UNW_V3) {
+        	fprintf(fp, "     F10: %05lx%016lx  ",
+                	eframe[P_f10_hi], eframe[P_f10_lo]);
+        	fprintf(fp, "  F11: %05lx%016lx\n",
+                	eframe[P_f11_hi], eframe[P_f11_lo]);
+	}
+
 	CRASHDEBUG_RESTORE();
         if (bt->debug)
                 CRASHDEBUG_SUSPEND(bt->debug);
@@ -2501,6 +2682,13 @@ ia64_post_init(void)
 	                                "unimpl_pa_mask", FAULT_ON_ERROR);
 		}
 	}
+
+        if (symbol_exists("ia64_init_stack") && !ms->ia64_init_stack_size) 
+		ms->ia64_init_stack_size = get_array_length("ia64_init_stack", 
+			NULL, 0);
+
+	if (DUMPFILE() && ia64_in_init_stack(SWITCH_STACK_ADDR(CURRENT_TASK())))
+		machdep->flags |= INIT;
 }
 
 /*
@@ -3046,7 +3234,8 @@ try_closest:
         if (strlen(buf)) {
                 if (retries) {
                         p = strstr(buf, ": ");
-			*p = NULLCHAR;
+			if (p)
+				*p = NULLCHAR;
                 }
                 fprintf(fp, "    %s\n", buf);
         } else {
