@@ -28,7 +28,7 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.15 $ $Date: 2004/05/24 14:59:08 $
+ * CVS: $Revision: 1.17 $ $Date: 2004/10/15 19:59:23 $
  */
 
 /*
@@ -64,11 +64,29 @@ static int hash_page(ulong);
 static int page_is_cached(void);
 static int page_is_hashed(long *);
 static int cache_page(void);
-static int get_closest_page(long *);
 
 struct lkcd_environment lkcd_environment = { 0 };
 struct lkcd_environment *lkcd = &lkcd_environment;
 
+ulonglong 
+fix_lkcd_address(ulonglong addr)
+{
+    int i; 
+    ulong offset;
+
+    for (i = 0; i < lkcd->fix_addr_num; i++) {
+	if ( (addr >=lkcd->fix_addr[i].task) && 
+		(addr <= lkcd->fix_addr[i].task + STACKSIZE())){
+
+	    offset = addr - lkcd->fix_addr[i].task;
+	    addr = lkcd->fix_addr[i].saddr + offset;
+	}
+    }
+
+    return addr;
+}
+
+
 /*
  *  Each version has its own dump initialization.
  */
@@ -575,16 +593,26 @@ lkcd_memory_dump(FILE *fp)
 			    i, sp->pg_addr, sp->pg_bufptr, sp->pg_hit_count);
         }
 
-	lkcd_print("mb_hdr_offsets[%3ld]: \n", lkcd->benchmark_pages);
+	if (lkcd->mb_hdr_offsets) {
+		lkcd_print("mb_hdr_offsets[%3ld]: \n", lkcd->benchmark_pages);
 
-	for (i = 0; i < lkcd->benchmark_pages; i += 8) {
-		lkcd_print("  [%3d]", i);
-		c = 0;
-		while ((c < 8) && ((i+c) < lkcd->benchmark_pages)) {
-			lkcd_print(" %8lx", lkcd->mb_hdr_offsets[i+c]);
-			c++;
+		for (i = 0; i < lkcd->benchmark_pages; i += 8) {
+			lkcd_print("  [%3d]", i);
+			c = 0;
+			while ((c < 8) && ((i+c) < lkcd->benchmark_pages)) {
+				lkcd_print(" %8lx", lkcd->mb_hdr_offsets[i+c]);
+				c++;
+			}
+			lkcd_print("\n");
 		}
-		lkcd_print("\n");
+	} else {
+		lkcd_print("  mb_hdr_offsets: NA\n");
+	}
+
+	if (lkcd->zones) {
+		lkcd_print("       num_zones: %d / %d\n", lkcd->num_zones,
+				lkcd->max_zones);
+		lkcd_print("   zoned_offsets: %ld\n", lkcd->zoned_offsets);
 	}
 
 	lkcd_print("  dumpfile_index: %s\n", lkcd->dumpfile_index);
@@ -607,16 +635,16 @@ lkcd_speedo(void)
 
         switch (++i%4) {
         case 0:
-                printf("|\b");
+                lkcd_print("|\b");
                 break;
         case 1:
-                printf("\\\b");
+                lkcd_print("\\\b");
                 break;
         case 2:
-                printf("-\b");
+                lkcd_print("-\b");
                 break;
         case 3:
-                printf("/\b");
+                lkcd_print("/\b");
                 break;
         }
 	fflush(stdout);
@@ -639,13 +667,115 @@ lkcd_speedo(void)
  *
  *   (1) page_is_hashed() will check whether the page header offset is cached,
  *       and if so, will set up the page variable, and lseek to the header.
- *   (2) get_closest_page() will find the page header offset of the closest
- *       hashed or benchmarked page, set up the page variable, and lseek 
- *       to the page's header.
  *
  *  In either case above, the starting point for the page search is set up.
  *  Lastly, cache_page() stores the requested page's data.
  */
+
+static int
+save_offset(uint64_t paddr, off_t off)
+{
+	uint64_t zone, page;
+	int ii, ret;
+
+	zone = paddr & lkcd->zone_mask;
+
+	page = (paddr & ~lkcd->zone_mask) >> lkcd->page_shift;
+
+	if (lkcd->num_zones == 0) {
+		lkcd->zones = malloc(ZONE_ALLOC * sizeof(struct physmem_zone));
+		if (!lkcd->zones) {
+			return -1; /* This should be fatal */
+		}
+		BZERO(lkcd->zones, ZONE_ALLOC * sizeof(struct physmem_zone));
+
+		lkcd->max_zones = ZONE_ALLOC;
+
+		lkcd->zones[0].start = zone;
+		lkcd->zones[0].pages = malloc((ZONE_SIZE >> lkcd->page_shift) *
+					sizeof(struct page_desc));
+		if (!lkcd->zones[0].pages) {
+			return -1; /* this should be fatal */
+		}
+
+		BZERO(lkcd->zones[0].pages, (ZONE_SIZE >> lkcd->page_shift) *
+					sizeof(struct page_desc));
+		lkcd->num_zones++;
+	}
+
+	/* find the zone */
+	for (ii=0; ii < lkcd->num_zones; ii++) {
+		if (lkcd->zones[ii].start == zone) {
+			if (lkcd->zones[ii].pages[page].offset != 0) {
+			   if (lkcd->zones[ii].pages[page].offset != off) {
+				error(INFO, "conflicting page: zone %lld, "
+					"page %lld: %lld, %lld != %lld\n",
+					(unsigned long long)zone, 
+					(unsigned long long)page, 
+					(unsigned long long)paddr, 
+					(unsigned long long)off,
+					(unsigned long long)lkcd->zones[ii].pages[page].offset);
+			  	abort();
+			   }
+			   ret = 0;
+			} else {
+			   lkcd->zones[ii].pages[page].offset = off;
+			   ret = 1;
+			}
+			break;
+		}
+	}
+	if (ii == lkcd->num_zones) {
+		/* This is a new zone */
+		if (lkcd->num_zones < lkcd->max_zones) {
+			/* We have room for another one */
+			lkcd->zones[ii].start = zone;
+			lkcd->zones[ii].pages = malloc(
+					(ZONE_SIZE >> lkcd->page_shift) *
+					sizeof(struct page_desc));
+			if (!lkcd->zones[ii].pages) {
+				return -1; /* this should be fatal */
+			}
+
+			BZERO(lkcd->zones[ii].pages, 
+					(ZONE_SIZE >> lkcd->page_shift) *
+					sizeof(struct page_desc));
+			lkcd->zones[ii].pages[page].offset = off;
+			ret = 1;
+			lkcd->num_zones++;
+		} else {
+			lkcd_print("fixme, need to add more zones (ZONE_ALLOC)\n");
+			exit(1);
+		}
+	}
+
+	return ret;  /* 1 if the page is new */
+}
+		
+static off_t
+get_offset(uint64_t paddr)
+{
+	uint64_t zone, page;
+	int ii;
+
+	zone = paddr & lkcd->zone_mask;
+	page = (paddr % ZONE_SIZE) >> lkcd->page_shift;
+
+	if (lkcd->zones == 0) {
+		return 0;
+	}
+
+	/* find the zone */
+	for (ii=0; ii < lkcd->num_zones; ii++) {
+		if (lkcd->zones[ii].start == zone) {
+			return (lkcd->zones[ii].pages[page].offset);
+		}
+	}
+	return 0;
+}
+
+
+
 int
 lkcd_lseek(physaddr_t paddr)
 {
@@ -653,10 +783,10 @@ lkcd_lseek(physaddr_t paddr)
 	int err;
         int eof;
         void *dp;
-	int pass;
         long page = 0;
 	physaddr_t physaddr;
 	int seeked_to_page = 0;
+	off_t page_offset;
 
 	dp = lkcd->dump_page;
 
@@ -671,116 +801,53 @@ lkcd_lseek(physaddr_t paddr)
 		seeked_to_page = 1;
 	}
 
-	/* 
-	 * All of the pages offsets should have been saved, 
-	 * so if we have plenty of swap space we simply page 
-	 * in the offset and seek.
-	 */
-	if (!seeked_to_page && lkcd->page_offsets != NULL) {
-		long page_index;
-		off_t page_offset;
-		off_t seek_offset;
+	 /* Find the offset for this page, if known */
+    if ((page_offset = get_offset(paddr)) > 0) {
+	off_t seek_offset;
+	seek_offset = lseek(lkcd->fd, page_offset, SEEK_SET);
 
-		page_index = (lkcd->curpaddr >> lkcd->page_shift);
-		page_offset = lkcd->page_offsets[page_index];
-		if( page_offset > 0 ) {
-			seek_offset = lseek(lkcd->fd, page_offset, SEEK_SET);
-			if (seek_offset == page_offset) {
-				seeked_to_page = 1;
-				page = page_index;
-			}
-		}
+	if (seek_offset == page_offset) {
+	    seeked_to_page = 1;
+	    page = 0; /* page doesn't make any sense */
 	}
-	if (seeked_to_page) {
-		err = lkcd_load_dump_page_header(dp, page);
-		if (err == LKCD_DUMPFILE_OK) {
-			return(cache_page());
-		}
-	}	
-		
+    }
 
-	/*
-	 * We haven't seen this page yet or we didn't have
-	 * space for lkcd->page_offsets[].
-	 */
-	for (pass = 1; pass <= 3; pass++) {
-		switch (pass) {
-		case 1:
-			/* 
-			 * If we have lkcd->page_offsets,
-			 * then we know the offsets for
-			 * all offsets before our largest
-			 * seen. So we start there.
-			 */
-			if ( lkcd->page_offsets != NULL ) {
-				lseek(lkcd->fd, lkcd->page_offset_max, SEEK_SET);
-				page = lkcd->page_index_max;
-                        } else {
-                                continue;
-                        }
-			break;
 
-		case 2:
-			/* 
-			 * We don't have lkcd->page_offsets,
-			 * but we might get lucky by looking
-			 * for the page next to a neighboring
-			 * page.
-			 */
-			if (!get_closest_page(&page)) {
-	               		continue;
-			}
-			break;
-
-		case 3:
-			/* 
-			 * We don't have lkcd->page_offsets, so
-			 * we have to use brute force and search
-			 * from the beginning.
-			 */
-			lseek(lkcd->fd, LKCD_OFFSET_TO_FIRST_PAGE, SEEK_SET);
-			page = 0;
-			break;
-		}
-	
-	        for (i = page, eof = FALSE; !eof; i++) {
-
-			if( (i%128) == 0) {
-				lkcd_speedo();
-			}
-	
-	                switch (lkcd_load_dump_page_header(dp, i))
-	                {
-	                case LKCD_DUMPFILE_OK:
-				break;
-	
-	                case LKCD_DUMPFILE_END:
-	                case LKCD_DUMPFILE_EOF:
-				eof = TRUE;
-	                        continue;
-	                }
-	
-	        	physaddr = lkcd->get_dp_flags() & 
-			    (LKCD_DUMP_MCLX_V0|LKCD_DUMP_MCLX_V1) ?
-	                    (lkcd->get_dp_address() - lkcd->kvbase) << lkcd->page_shift:
-	                    lkcd->get_dp_address() - lkcd->kvbase;
-	
-			if (physaddr == lkcd->curpaddr) {
-				/* 
-				 * Update the mmap'd index file cache if it exists 
-				 * and return with the cached page.
-				 */
-                                if (lkcd->page_offsets != NULL) {
-                                        msync(&lkcd->page_offsets[0],
-                                              	(lkcd->page_index_max *
-						sizeof(off_t)), MS_ASYNC);
-                                }
-				return(cache_page());
-			}
-	
-	        	lseek(lkcd->fd, lkcd->get_dp_size(), SEEK_CUR);
-		}
+    if (seeked_to_page) {
+	err = lkcd_load_dump_page_header(dp, page);
+	if (err == LKCD_DUMPFILE_OK) {
+	    return(cache_page());
 	}
+    }	
+
+    /* We have to grind through some more of the dump file */
+    lseek(lkcd->fd, lkcd->page_offset_max, SEEK_SET);
+    eof = FALSE;
+    while (!eof) {
+	if( (i%2048) == 0) {
+	    lkcd_speedo();
+	}
+
+	switch (lkcd_load_dump_page_header(dp, page))
+	{
+	    case LKCD_DUMPFILE_OK:
+		break;
+
+	    case LKCD_DUMPFILE_EOF:
+		eof = TRUE;
+		continue;
+	}
+
+	physaddr = lkcd->get_dp_flags() & 
+	    (LKCD_DUMP_MCLX_V0|LKCD_DUMP_MCLX_V1) ?
+	    (lkcd->get_dp_address() - lkcd->kvbase) << lkcd->page_shift:
+	    lkcd->get_dp_address() - lkcd->kvbase;
+
+	if (physaddr == lkcd->curpaddr) {
+	    return(cache_page());
+	}
+	lseek(lkcd->fd, lkcd->get_dp_size(), SEEK_CUR);
+    }
 
 	return FALSE;
 }
@@ -915,61 +982,6 @@ page_is_hashed(long *pp)
 
 }
 
-/*
- *  Given lkcd->curpaddr, first find the closest lower hashed page, if any,
- *  and then find its associated benchmark page.  Take whichever is closer,
- *  lseek to it, and return its page number.
- */
-static int
-get_closest_page(long *pp)
-{
-        int i;
-        struct page_hash_entry *phe, *phe_closest;
-        uint64_t curdiff;
-	long mb, page, mb_page, phe_page;
-
-        curdiff = lkcd->curpaddr;
-        phe_closest = NULL;
-
-        for (i = 0; i < LKCD_PAGE_HASH; i++) {
-                for (phe = &lkcd->page_hash[i]; LKCD_VALID_PAGE(phe->pg_flags);
-                     phe = phe->next) {
-                        if (phe->pg_addr < lkcd->curpaddr) {
-                                if ((lkcd->curpaddr - phe->pg_addr) < curdiff) {
-                                        curdiff = lkcd->curpaddr - phe->pg_addr;
-                                        phe_closest = phe;
-                                }
-                        }
-                        if (!phe->next)
-                                break;
-                }
-        }
-
-	page = (long)(lkcd->curpaddr >> lkcd->page_shift);
-
-	if ((mb = LKCD_PAGE_MEGABYTE(page)) >= lkcd->benchmark_pages)
-		mb = lkcd->benchmark_pages - 1;
-
-	for (/* mb setup above */; !lkcd->mb_hdr_offsets[mb]; mb--) {
-		if (mb < 0) 
-			return FALSE;
-        }
-
-	mb_page = mb * LKCD_PAGES_PER_MEGABYTE();
-	phe_page = phe_closest ? 
-		(long)(phe_closest->pg_addr >> lkcd->page_shift) : 0;
-
-	if (phe_page > mb_page) {
-		*pp = phe_page;
-                 lseek(lkcd->fd, phe_closest->pg_hdr_offset, SEEK_SET);
-	} else {
-		*pp = mb_page;
-        	lseek(lkcd->fd, lkcd->mb_hdr_offsets[mb], SEEK_SET);
-	}
-
-	return TRUE;
-}
-
 /*
  *  The caller stores the incoming page's page header offset in 
  *  lkcd->curhdroffs.
@@ -1232,7 +1244,10 @@ lkcd_load_dump_page_header(void *dp, ulong page)
 	uint32_t dp_flags;
 	uint64_t dp_address, physaddr;
 	off_t page_offset;
+	int ret;
 
+
+	/* This is wasted effort */
         page_offset = lkcd->curhdroffs = lseek(lkcd->fd, 0, SEEK_CUR);
 
         if (read(lkcd->fd, dp, lkcd->page_header_size) != 
@@ -1267,43 +1282,20 @@ lkcd_load_dump_page_header(void *dp, ulong page)
 		(dp_address - lkcd->kvbase) << lkcd->page_shift : 
         	dp_address - lkcd->kvbase;
 
-	/*
-	 * Update page_offsets if possible and needed.
-	 */
-	if(lkcd->page_offsets != NULL) {
-		long page_index;
 
-		page_index = physaddr >> lkcd->page_shift;
-		if (page_index > lkcd->memory_pages) {
-			abort();
-		}
-		if ( lkcd->page_offsets[page_index] == 0 ) {
-			if (lkcd->page_offset_max > page_offset) {
-				/* 
-				 * Can't Happen - No holes allowed in page_offsets[]:
-				 *
-				 * It cost us a lot to seek and read the page headers.
-				 * Once we read it we never dispose of it. So if we
-				 * are looking for a page and we don't have the offset
-				 * for it then we haven't seen it and need to continue
-				 * reading the dump.
-				 */
-				abort();
-			}
-			lkcd->page_offsets[page_index] = page_offset;
-			lkcd->page_index_max = page_index;
-			lkcd->page_offset_max = page_offset;
-		} else {
-			if (lkcd->page_offsets[page_index] != lkcd->curhdroffs) {
-				abort();
-			}
-		}
+	if ((ret = save_offset(physaddr, page_offset)) < 0) {
+	    return LKCD_DUMPFILE_EOF; /* really an error */
+	} 
+
+	lkcd->zoned_offsets += ret;  /* return = 0 if already known */
+
+	if (page_offset > lkcd->page_offset_max) {
+	    /* doesn't this mean I have to re-read this dp? */
+	    lkcd->page_offset_max = page_offset;
 	}
 
-        if (MEGABYTE_ALIGNED(physaddr) && !LKCD_BENCHMARKS_DONE()) 
-                set_mb_benchmark((ulong)(physaddr >> lkcd->page_shift));
 
-        return LKCD_DUMPFILE_OK;
+	return LKCD_DUMPFILE_OK;
 }
 
 /*
