@@ -24,7 +24,7 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.3 $ $Date: 2004/01/28 17:09:24 $
+ * CVS: $Revision: 1.4 $ $Date: 2004/10/14 15:56:02 $
  */
 
 #define LKCD_COMMON
@@ -85,14 +85,18 @@ lkcd_dump_init_v2_v3(FILE *fp, int fd)
 	lkcd->bits = sizeof(long) * 8;
         lkcd->benchmark_pages = (dh->dh_num_pages/LKCD_PAGES_PER_MEGABYTE())+1;
 	lkcd->total_pages = dh->dh_num_pages;
-        if ((lkcd->mb_hdr_offsets = (off_t *)malloc
-            (lkcd->benchmark_pages * sizeof(off_t))) == NULL)
-                return FALSE;
+
+	lkcd->zone_shift = ffs(ZONE_SIZE) - 1;
+	lkcd->zone_mask = ~(ZONE_SIZE - 1);
+	lkcd->num_zones = 0;
+	lkcd->max_zones = 0;
+	lkcd->zoned_offsets = 0;
+
 	lkcd->get_dp_flags = get_dp_flags_v2_v3;
 	lkcd->get_dp_address = get_dp_address_v2_v3;
+	lkcd->get_dp_size = get_dp_size_v2_v3;
 	lkcd->compression = LKCD_DUMP_COMPRESS_RLE;
         lkcd->page_header_size = sizeof(dump_page_t);
-	BZERO(lkcd->mb_hdr_offsets, lkcd->benchmark_pages * sizeof(off_t));
 
         lseek(lkcd->fd, LKCD_OFFSET_TO_FIRST_PAGE, SEEK_SET);
 
@@ -162,7 +166,6 @@ lkcd_dump_init_v2_v3(FILE *fp, int fd)
 		pgcnt : dh->dh_num_pages;
 	lkcd->panic_task = (ulong)dh->dh_current_task;
 	lkcd->panic_string = (char *)&dh->dh_panic_string[0];
-	lkcd->get_dp_size = get_dp_size_v2_v3;
 
 	if (dh->dh_version & LKCD_DUMP_MCLX_V1) 
 		mclx_cache_page_headers_v3();
