@@ -15,7 +15,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * CVS: $Revision: 1.8 $ $Date: 2004/05/24 14:30:28 $
+ * CVS: $Revision: 1.9 $ $Date: 2004/10/14 15:56:02 $
  */
 
 #define LKCD_COMMON
@@ -55,11 +55,17 @@ lkcd_dump_init_v7(FILE *fp, int fd, char *dumpfile)
 	    sizeof(dump_header_t))
 		return FALSE;
 
+
         lkcd->dump_page = dp;
         lkcd->dump_header = dh;
 	if (lkcd->debug) 
 		dump_lkcd_environment(LKCD_DUMP_HEADER_ONLY);
 
+#ifdef IA64
+	if ( (fix_addr_v7(fd) == -1) )
+	    return FALSE;
+#endif
+
 	/*
 	 *  Allocate and clear the benchmark offsets, one per megabyte.
 	 */
@@ -72,6 +78,19 @@ lkcd_dump_init_v7(FILE *fp, int fd, char *dumpfile)
 	 * REMIND: dh_memory_size should be in physical pages and seems to be wrong.
 	 *         pad by two for now; 3DFE8 should be 40000.
 	 */
+
+	lkcd->memory_pages = dh->dh_memory_size;
+	lkcd->page_offsets = 0;
+	lkcd->ifd = -1;
+	lkcd->dumpfile_index = NULL;
+
+        /* Keep from getting unused warnings */
+	dump_index_size = 0; 
+	dump_index_created = 0;
+	strcpy(dumpfile_index_name, dumpfile);
+	ifd = 0;
+
+#ifdef LKCD_INDEX_FILE
 	lkcd->memory_pages = (dh->dh_memory_size * (getpagesize()/lkcd->page_size)) * 2;
 	dump_index_size = (lkcd->memory_pages * sizeof(off_t));	
 	lkcd->page_offsets = 0;
@@ -81,7 +100,7 @@ lkcd_dump_init_v7(FILE *fp, int fd, char *dumpfile)
 	if( ifd < 0 ) {
 		int err;
 
-		ifd = open(lkcd->dumpfile_index, (O_RDWR | O_CREAT));
+		ifd = open(lkcd->dumpfile_index, (O_RDWR | O_CREAT), 0644);
 
 		if (ifd > 0) {
 			err = ftruncate(ifd, dump_index_size);
@@ -106,41 +125,19 @@ lkcd_dump_init_v7(FILE *fp, int fd, char *dumpfile)
 		}
 	}
 	lkcd->ifd = ifd;
+#endif 
 
-	if (lkcd->page_offsets == 0 ) {
-		lkcd->page_offsets = malloc(lkcd->memory_pages * sizeof(off_t));
-		dump_index_created++;
-	}
-	if(lkcd->page_offsets) {
-		lkcd->page_offset_max = 0;
-		lkcd->page_index_max = -1;
-		if(dump_index_created) {
-			bzero(lkcd->page_offsets, lkcd->memory_pages * sizeof(off_t));
-		} else {
-			/* Find how far we have read into the dumpfile */
-			for(i = 0; i < lkcd->memory_pages; i++) {
-				if (lkcd->page_offsets[i] > lkcd->page_offset_max) {
-					lkcd->page_offset_max = lkcd->page_offsets[i];
-					lkcd->page_index_max = i;
-				}
-			}
-		}
-	} else {
-		/* 
-		 * page_offsets[]  too big we will just live without it.
-		 */
-		printf("Failed to allocate array for saved offsets.\n");
+	lkcd->zone_shift = ffs(ZONE_SIZE) - 1;
+	lkcd->zone_mask = ~(ZONE_SIZE - 1);
+	lkcd->num_zones = 0;
+	lkcd->max_zones = 0;
+	lkcd->zoned_offsets = 0;
 
-		printf("Dump access wll be extremely slow\n");
-	}
-        if ((lkcd->mb_hdr_offsets = (off_t *)malloc
-            (lkcd->benchmark_pages * sizeof(off_t))) == NULL)
-                return FALSE;
 	lkcd->get_dp_flags = get_dp_flags_v7;
 	lkcd->get_dp_address = get_dp_address_v7;
+	lkcd->get_dp_size = get_dp_size_v7;
    	lkcd->compression = dh->dh_dump_compress; 
         lkcd->page_header_size = sizeof(dump_page_t);
-	BZERO(lkcd->mb_hdr_offsets, lkcd->benchmark_pages * sizeof(off_t));
 
         lseek(lkcd->fd, LKCD_OFFSET_TO_FIRST_PAGE, SEEK_SET);
 
@@ -213,7 +210,6 @@ lkcd_dump_init_v7(FILE *fp, int fd, char *dumpfile)
 		pgcnt : dh->dh_num_pages;
 	lkcd->panic_task = (ulong)dh->dh_current_task;
 	lkcd->panic_string = (char *)&dh->dh_panic_string[0];
-	lkcd->get_dp_size = get_dp_size_v7;
 
 	if (dh->dh_version & LKCD_DUMP_MCLX_V1) 
 		mclx_cache_page_headers_v7();
@@ -498,3 +494,4 @@ mclx_cache_page_headers_v7(void)
 		set_mb_benchmark((granularity * (i+1))/lkcd->page_size);
 	}
 }
+
