@@ -25,7 +25,7 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.3 $ $Date: 2004/01/28 17:08:42 $
+ * CVS: $Revision: 1.4 $ $Date: 2004/07/13 20:17:04 $
  */
 
 
@@ -46,7 +46,9 @@
 #ifndef MCLX
 #include <linux/utsname.h>              /* for utsname structure            */
 #endif
+#ifndef IA64
 #include <asm/ptrace.h>                 /* for pt_regs                      */
+#endif
 
 /* necessary header definitions in all cases */
 #define DUMP_KIOBUF_NUMBER  0xdeadbeef  /* special number for kiobuf maps   */
@@ -124,7 +126,9 @@ typedef struct _dump_header_s {
 	struct new_utsname   dh_utsname;
 
 	/* the dump registers */
+#ifndef IA64
 	struct pt_regs       dh_regs;
+#endif
 
 	/* the address of the current task */
 	struct task_struct  *dh_current_task;
@@ -153,8 +157,10 @@ typedef struct _dump_page_s {
 
 #endif /* CONFIG_VMDUMP */
 
+#ifdef __KERNEL__
 extern void dump_init(uint64_t, uint64_t);
 extern void dump_open(char *);
 extern void dump_execute(char *, struct pt_regs *);
+#endif
 
 #endif /* _VMDUMP_H */
