@@ -24,7 +24,7 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.4 $ $Date: 2004/05/11 15:42:47 $
+ * CVS: $Revision: 1.6 $ $Date: 2004/08/19 19:03:09 $
  */
 
 
@@ -101,14 +101,16 @@ typedef struct _dump_header_asm_s {
 
 #endif /* X86 || PPC */
 
-#if defined(ALPHA) || defined(IA64) || defined(X86_64)
+#if defined(ALPHA) || defined(IA64) || defined(X86_64) || defined(PPC64)
 
 /* 
  *  Plug in the real ../arch/alpha/vmdump.h when available.  For now the
  *  data here are just placeholders...
  */
 
+#ifndef IA64
 #include <asm/ptrace.h>                          /* for pt_regs             */
+#endif
 
 /* definitions */
 #define DUMP_ASM_MAGIC_NUMBER     0xdeaddeadULL  /* magic number            */
@@ -133,7 +135,9 @@ typedef struct _dump_header_asm_s {
         uint32_t             dha_eip;
 
         /* the dump registers */
+#ifndef IA64
         struct pt_regs       dha_regs;
+#endif
 
 } dump_header_asm_t;
 
@@ -235,8 +239,10 @@ typedef struct _dump_page_s {
 
 #endif /* CONFIG_VMDUMP */
 
+#ifdef __KERNEL__
 extern void dump_init(uint64_t, uint64_t);
 extern void dump_open(char *);
 extern void dump_execute(char *, struct pt_regs *);
+#endif
 
 #endif /* _VMDUMP_H */
