@@ -27,14 +27,14 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.41 $ $Date: 2004/06/22 16:46:23 $
+ * CVS: $Revision: 1.44 $ $Date: 2004/11/22 23:07:24 $
  */
 
 #include "defs.h"
 #include <curses.h>
 #include <getopt.h>
 
-static void setup_environment(char *);
+static void setup_environment(int, char **);
 static int is_external_command(void);
 static int is_builtin_command(void);
 static int is_input_file(void);
@@ -48,7 +48,10 @@ static struct option long_options[] = {
         {"help", 0, 0, 0},
 	{"data_debug", 0, 0, 0},
 	{"no_data_debug", 0, 0, 0},
+	{"no_crashrc", 0, 0, 0},
 	{"no_kmem_cache", 0, 0, 0},
+	{"readnow", 0, 0, 0},
+	{"smp", 0, 0, 0},
 	{"machdep", 1, 0, 0},
 	{"version", 0, 0, 0},
 	{"buildinfo", 0, 0, 0},
@@ -60,7 +63,7 @@ main(int argc, char **argv)
 {
 	int c, option_index;
 
-	setup_environment(argv[0]);
+	setup_environment(argc, argv);
 
 	/* 
 	 *  Get and verify command line options.
@@ -109,6 +112,14 @@ main(int argc, char **argv)
 			    "no_kmem_cache")) 
 				vt->flags |= KMEM_CACHE_UNAVAIL;
 
+		        if (STREQ(long_options[option_index].name, 
+			    "readnow")) 
+				pc->flags |= READNOW;
+
+		        if (STREQ(long_options[option_index].name, 
+			    "smp")) 
+				kt->flags |= SMP;
+
 		        if (STREQ(long_options[option_index].name, 
 			    "machdep")) 
 				machdep->cmdline_arg = optarg;
@@ -601,11 +612,15 @@ cmd_mach(void)
 
 
 static void
-setup_environment(char *program)
+setup_environment(int argc, char **argv)
 {
+	int i;
 	char *p1;
 	char buf[BUFSIZE];
 	FILE *afp;
+	char *program;
+
+	program = argv[0];
 
 	/*
 	 *  Program output typically goes via "fprintf(fp, ...)", but the 
@@ -683,6 +698,10 @@ setup_environment(char *program)
 	 *  Resolve $HOME .rc file first, then the one in the local directory.
          *  Note that only "set" and "alias" commands are done at this time.
 	 */
+	for (i = 1; i < argc; i++)
+		if (STREQ(argv[i], "--no_crashrc"))
+			pc->flags |= NOCRASHRC; 
+
 	alias_init(NULL);
 
 	if ((p1 = getenv("HOME"))) {
@@ -693,7 +712,7 @@ setup_environment(char *program)
 		} else
 			strcpy(pc->home, p1);
 	        sprintf(buf, "%s/.%src", pc->home, pc->program_name);
-	        if (file_exists(buf, NULL)) {
+	        if (!(pc->flags & NOCRASHRC) && file_exists(buf, NULL)) {
 	                if ((afp = fopen(buf, "r")) == NULL)
 	                        error(INFO, "cannot open %s: %s\n",
 	                                buf, strerror(errno));
@@ -706,7 +725,7 @@ setup_environment(char *program)
 	}
 
         sprintf(buf, ".%src", pc->program_name);
-	if (file_exists(buf, NULL)) {
+	if (!(pc->flags & NOCRASHRC) && file_exists(buf, NULL)) {
 		if ((afp = fopen(buf, "r")) == NULL)
                         error(INFO, "cannot open %s: %s\n",
 				buf, strerror(errno));
@@ -877,6 +896,12 @@ dump_program_context(void)
        if (pc->flags & VERSION_QUERY)
                 sprintf(&buf[strlen(buf)],
                         "%sVERSION_QUERY", others++ ? "|" : "");
+       if (pc->flags & READNOW)
+                sprintf(&buf[strlen(buf)],
+                        "%sREADNOW", others++ ? "|" : "");
+       if (pc->flags & NOCRASHRC)
+                sprintf(&buf[strlen(buf)],
+                        "%sNOCRASHRC", others++ ? "|" : "");
 
 	if (pc->flags)
 		strcat(buf, ")");
