@@ -12,7 +12,7 @@
  *
  * Author: David Anderson
  *
- * CVS: $Revision: 1.25 $ $Date: 2004/06/22 16:43:12 $
+ * CVS: $Revision: 1.33 $ $Date: 2005/02/17 15:48:56 $
  */
 
 #include "defs.h"
@@ -111,13 +111,12 @@ is_netdump(char *file, ulong source)
 		default:
                 	goto bailout;
 		}
-               nd->flags |= NETDUMP_ELF32;
+                nd->flags |= NETDUMP_ELF32;
                 load32 = (Elf32_Phdr *)
                         &header[sizeof(Elf32_Ehdr)+sizeof(Elf32_Phdr)];
                 size = (size_t)load32->p_offset;
 	} else if (STRNEQ(elf64->e_ident, ELFMAG) &&
 	    (elf64->e_ident[EI_CLASS] == ELFCLASS64) &&
-	    (elf64->e_ident[EI_DATA] == ELFDATA2LSB) &&
 	    (elf64->e_ident[EI_VERSION] == EV_CURRENT) &&
 	    (elf64->e_type == ET_CORE) &&
 	    (elf64->e_version == EV_CURRENT) &&
@@ -125,14 +124,26 @@ is_netdump(char *file, ulong source)
 		switch (elf64->e_machine)
 		{
 		case EM_IA_64:
-			if (STREQ(MACHINE_TYPE, "IA64"))
+			if ((elf64->e_ident[EI_DATA] == ELFDATA2LSB) &&
+				STREQ(MACHINE_TYPE, "IA64"))
 				break;
+			else
+				goto bailout;
+
 		case EM_PPC64:
-			if (STREQ(MACHINE_TYPE, "PPC64"))
+			if ((elf64->e_ident[EI_DATA] == ELFDATA2MSB) &&
+				STREQ(MACHINE_TYPE, "PPC64"))
 				break;
+			else
+				goto bailout;
+
 		case EM_X86_64:
-			if (STREQ(MACHINE_TYPE, "X86_64"))
+			if ((elf64->e_ident[EI_DATA] == ELFDATA2LSB) &&
+				STREQ(MACHINE_TYPE, "X86_64"))
 				break;
+			else
+				goto bailout;
+
 		default:
 			goto bailout;
 		}
@@ -224,12 +235,11 @@ read_netdump(int fd, void *bufptr, int cnt, ulong addr, physaddr_t paddr)
 
         offset = (off_t)paddr + (off_t)nd->header_size;
 
-        if (lseek(nd->ndfd, offset, SEEK_SET) != offset)
+        if (lseek(nd->ndfd, offset, SEEK_SET) == -1)
                 return SEEK_ERROR;
 
         if (read(nd->ndfd, bufptr, cnt) != cnt)
                 return READ_ERROR;
-
         return cnt;
 }
 
@@ -343,7 +353,6 @@ get_netdump_panic_task(void)
 	return nd->task_struct;
 #else
 	int i;
-        Elf32_Nhdr *note32;
         size_t len;
 	char *user_regs;
 	ulong ebp, esp, task;
@@ -354,8 +363,8 @@ get_netdump_panic_task(void)
 	if (nd->task_struct)
 		return nd->task_struct;
 
-        if (nd->elf32->e_machine == EM_386) {
-	        note32 = (Elf32_Nhdr *)
+        if (nd->elf32 && nd->elf32->e_machine == EM_386) {
+	        Elf32_Nhdr *note32 = (Elf32_Nhdr *)
 			((char *)nd->elf32 + nd->notes32->p_offset);
 		
 	        len = sizeof(Elf32_Nhdr);
@@ -392,7 +401,35 @@ get_netdump_panic_task(void)
                                       return task;
                         }
                 }
-	}
+	} else if (nd->elf64) {
+	        Elf64_Nhdr *note64 = (Elf64_Nhdr *)
+			((char *)nd->elf64 + nd->notes64->p_offset);
+		
+	        len = sizeof(Elf64_Nhdr);
+	        len = roundup(len + note64->n_namesz, 4);
+		user_regs = (char *)((char *)note64 + len +
+			MEMBER_OFFSET("elf_prstatus", "pr_reg"));
+		if (nd->elf64->e_machine == EM_PPC64) {
+			/*
+			 * Get the GPR1 register value.
+			 */
+			esp = *(ulong *)((char *)user_regs + 8);
+			if (CRASHDEBUG(1)) 
+				fprintf(fp, 
+			    	"get_netdump_panic_task: esp: %lx\n", esp);
+			if (IS_KVADDR(esp)) {
+				task = stkptr_to_task(esp);
+				if (CRASHDEBUG(1))
+					fprintf(fp, 
+			    		"get_netdump_panic_task: esp: %lx -> task: %lx\n",
+						esp, task);
+				for (i = 0; task && (i < NR_CPUS); i++) {
+					if (task == tt->active_set[i]) 
+						return task;
+				}
+			}
+		}
+	} 
 
 	if (CRASHDEBUG(1))
 		fprintf(fp, "get_netdump_panic_task: returning NO_TASK\n");
@@ -942,7 +979,7 @@ dump_Elf32_Nhdr(Elf32_Off offset, int store_address)
         BZERO(buf, BUFSIZE);
         ptr = (char *)note + sizeof(Elf32_Nhdr);
         BCOPY(ptr, buf, note->n_namesz);
-        netdump_print("(\"%s\")\n", ptr);
+        netdump_print("(\"%s\")\n", buf);
 
         netdump_print("               n_descsz: %ld\n", note->n_descsz);
         netdump_print("                 n_type: %ld ", note->n_type);
@@ -1009,7 +1046,7 @@ dump_Elf64_Nhdr(Elf64_Off offset, int store_address)
         BZERO(buf, BUFSIZE);
         ptr = (char *)note + sizeof(Elf64_Nhdr);
         BCOPY(ptr, buf, note->n_namesz);
-        netdump_print("(\"%s\")\n", ptr);
+        netdump_print("(\"%s\")\n", buf);
 
         netdump_print("               n_descsz: %ld\n", note->n_descsz);
         netdump_print("                 n_type: %ld ", note->n_type);
@@ -1116,7 +1153,7 @@ get_netdump_regs_x86_64(struct bt_info *bt, ulong *ripp, ulong *rspp)
         if (is_task_active(bt->task)) 
                 bt->flags |= BT_DUMPFILE_SEARCH;
 
-	if (bt->task == tt->panic_task) {
+	if (VALID_STRUCT(user_regs_struct) && (bt->task == tt->panic_task)) {
                 note = (Elf64_Nhdr *)nd->nt_prstatus;
 
                 len = sizeof(Elf64_Nhdr);
@@ -1168,6 +1205,7 @@ retry:
 		sym = closest_symbol(*up);
 		if (STREQ(sym, "netconsole_netdump") || 
 		    STREQ(sym, "netpoll_start_netdump") ||
+		    STREQ(sym, "start_disk_dump") ||
 		    STREQ(sym, "disk_dump")) {
 			*eip = *up;
 			*esp = search ?
@@ -1274,6 +1312,17 @@ next_sysrq:
 static void
 get_netdump_regs_ppc64(struct bt_info *bt, ulong *eip, ulong *esp)
 {
-	error(FATAL, "get_netdump_regs_ppc64: TBD\n");
+	Elf64_Nhdr *note;
+	size_t len;
+
+	if (bt->task == tt->panic_task) {
+		note = (Elf64_Nhdr *)nd->nt_prstatus;
+
+		len = sizeof(Elf64_Nhdr);
+		len = roundup(len + note->n_namesz, 4);
+		bt->machdep = (void *)((char *)note + len + 
+			MEMBER_OFFSET("elf_prstatus", "pr_reg"));
+	}
+
 	machdep->get_stack_frame(bt, eip, esp);
 }
