@@ -21,7 +21,7 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.26 $ $Date: 2004/06/08 20:31:48 $
+ * CVS: $Revision: 1.28 $ $Date: 2004/09/02 21:13:01 $
  */ 
 #ifdef PPC
 #include "defs.h"
@@ -145,6 +145,8 @@ ppc_init(int when)
 		else
 			machdep->nr_irqs = 0;
 		machdep->hz = HZ;
+		if (THIS_KERNEL_VERSION >= LINUX(2,6,0))
+			machdep->hz = 1000;
 		break;
 
 	case POST_INIT:
@@ -1467,7 +1469,8 @@ try_closest:
         if (strlen(buf)) {
                 if (retries) {
                         p = strstr(buf, ": ");
-                        *p = NULLCHAR;
+			if (p)
+                        	*p = NULLCHAR;
                 }
                 fprintf(fp, "    %s\n", buf);
         } else {
