@@ -13,7 +13,7 @@
  *
  *    arch/ia64/kernel/unwind.c  (kernel-2.4.18-6.23)
  *
- *  CVS: $Revision: 1.28 $ $Date: 2004/06/15 11:58:28 $
+ *  CVS: $Revision: 1.34 $ $Date: 2005/02/07 21:02:44 $
  */
 
 #ifdef IA64
@@ -31,6 +31,10 @@
  *  unwind_v[123].o object files.
  */
 
+/*
+ *  2004-09-14 J. Nomura    Added OS_INIT handling
+ */
+
 /* #include <asm/ptrace.h>  can't include this -- it's changing over time! */
 
 #include "defs.h"
@@ -42,6 +46,7 @@ typedef unsigned long long u64;
 #define GATE_ADDR (0xa000000000000000 + PAGE_SIZE)
 #define CLEAR_SCRIPT_CACHE (TRUE)
 
+#define _ASM_IA64_FPU_H
 #include "unwind.h"
 #include "unwind_i.h"
 #include "rse.h"
@@ -55,6 +60,13 @@ static void verify_common_struct(char *, long);
 static void dump_unwind_table(struct unw_table *);
 static int unw_init_from_blocked_task(struct unw_frame_info *, 
 	struct bt_info *);
+static void unw_init_from_interruption(struct unw_frame_info *,
+	struct bt_info *, ulong, ulong);
+static int unw_switch_from_osinit_v1(struct unw_frame_info *,
+	struct bt_info *);
+static int unw_switch_from_osinit_v2(struct unw_frame_info *,
+	struct bt_info *);
+static unsigned long get_init_stack_ulong(unsigned long addr);
 static void unw_init_frame_info(struct unw_frame_info *, 
 	struct bt_info *, ulong);
 static int find_save_locs(struct unw_frame_info *);
@@ -109,7 +121,7 @@ static struct {
 
 	/* table of registers that prologues can save 
            (and order in which they're saved): */
-	const unsigned char save_order[8];
+	unsigned char save_order[8];
 
 	/* maps a preserved register index (preg_index) to corresponding 
            switch_stack offset: */
@@ -206,7 +218,8 @@ pt_regs_off_v2 (unsigned long reg)
                 off = pt_regs_offsets[reg];
 
         if (off < 0) {
-                error(INFO, "unwind: bad scratch reg r%lu\n", reg);
+		if (reg > 0)
+                	error(INFO, "unwind: bad scratch reg r%lu\n", reg);
                 off = 0;
         }
         return (unsigned long) off;
@@ -231,7 +244,7 @@ pt_regs_off (unsigned long reg)
 		off = struct_offset(struct pt_regs, r12) + 8*(reg - 12);
 	else if (reg <= 31)
 		off = struct_offset(struct pt_regs, r16) + 8*(reg - 16);
-	else
+	else if (reg > 0)
 		error(INFO, "unwind: bad scratch reg r%lu\n", reg);
 	return off;
 }
@@ -404,8 +417,8 @@ unw_access_gr_v3 (struct unw_frame_info *info, int regnum, unsigned long *val, c
 		else
 			*nat_addr &= ~nat_mask;
 	} else {
-		if ((GET_STACK_ULONG(nat_addr) & nat_mask) == 0) {
-			*val = GET_STACK_ULONG(addr);
+		if ((IA64_GET_STACK_ULONG(nat_addr) & nat_mask) == 0) {
+			*val = IA64_GET_STACK_ULONG(addr);
 			*nat = 0;
 		} else {
 			*val = 0;	/* if register is a NaT, *addr may contain kernel data! */
@@ -455,7 +468,7 @@ unw_access_br_v3 (struct unw_frame_info *info, int regnum, unsigned long *val, i
 	if (write)
 		*addr = *val;
 	else
-		*val = GET_STACK_ULONG(addr);
+		*val = IA64_GET_STACK_ULONG(addr);
 	return 0;
 }
 
@@ -693,7 +706,7 @@ unw_access_ar_v3 (struct unw_frame_info *info, int regnum, unsigned long *val, i
 			*info->cfm_loc =
 				(*info->cfm_loc & ~(0x3fUL << 52)) | ((*val & 0x3f) << 52);
 		else
-			*val = (GET_STACK_ULONG(info->cfm_loc) >> 52) & 0x3f;
+			*val = (IA64_GET_STACK_ULONG(info->cfm_loc) >> 52) & 0x3f;
 		return 0;
 
 	      case UNW_AR_FPSR:
@@ -737,7 +750,7 @@ unw_access_ar_v3 (struct unw_frame_info *info, int regnum, unsigned long *val, i
 	if (write)
 		*addr = *val;
 	else
-		*val = GET_STACK_ULONG(addr);
+		*val = IA64_GET_STACK_ULONG(addr);
 	return 0;
 }
 
@@ -762,7 +775,7 @@ unw_access_pr_v3 (struct unw_frame_info *info, unsigned long *val, int write)
 	if (write)
 		*addr = *val;
 	else
-		*val = GET_STACK_ULONG(addr);
+		*val = IA64_GET_STACK_ULONG(addr);
 	return 0;
 }
 
@@ -1392,11 +1405,15 @@ unwind_init_v3(void)
 		machdep->machspec->unw_tables_offset =
 			 req->member_offset/BITS_PER_BYTE;
 
+		if (get_symbol_type("unw", "r0", req) != TYPE_CODE_UNDEF) 
+			machdep->flags |= UNW_R0;
+
 		verify_unw_member("save_order", 
 			struct_offset(struct unw, save_order));
 		verify_unw_member("sw_off", struct_offset(struct unw, sw_off));
 		verify_unw_member("preg_index", 
 			struct_offset(struct unw, preg_index));
+
         	if (get_symbol_type("unw", "pt_regs_offsets", req) 
 			== TYPE_CODE_ARRAY) {
 			machdep->machspec->unw_pt_regs_offsets =
@@ -1412,7 +1429,7 @@ unwind_init_v3(void)
 				struct_offset(struct unw, kernel_table));
 
 		if (!load_unw_table(CLEAR_SCRIPT_CACHE)) {
-        		error(WARNING, "cannot read kernel unw table\n");
+        		error(WARNING, "unwind_init: cannot read kernel unw table\n");
                 	machdep->flags |= UNW_OUT_OF_SYNC;
 		}
 
@@ -1448,6 +1465,8 @@ verify_unw_member(char *member, long loffs)
         } else {
                 koffs = (req->member_offset/BITS_PER_BYTE) -
                 	machdep->machspec->unw_tables_offset;
+		if (machdep->flags & UNW_R0)
+			koffs -= sizeof(unsigned long);
                 if (koffs != loffs) {
                         error(WARNING, 
 			    "unw.%s offset differs: %ld (local: %d)\n",
@@ -1518,20 +1537,56 @@ load_unw_table(int clear_cache)
 	if (machdep->flags & UNW_READ)
 		return TRUE;
 
-	if (!readmem(symbol_value("unw")+ms->unw_tables_offset, 
-	    KVADDR, &unw, sizeof(struct unw), "unw", RETURN_ON_ERROR|QUIET)) 
-		return FALSE;
+	if (machdep->flags & UNW_R0) {
+		struct unw *unw_temp, *up;
+
+		unw_temp = (struct unw *)GETBUF(sizeof(struct unw) * 2);
+		up = unw_temp;
+
+		if (!readmem(symbol_value("unw")+ms->unw_tables_offset, 
+	    	    KVADDR, up, 
+		    sizeof(struct unw) + sizeof(struct unw_table *), 
+		    "unw", RETURN_ON_ERROR|QUIET)) 
+			return FALSE;
+
+		unw.tables = up->tables;
+
+		/*
+		 *  Bump the "up" pointer by 8 to account for the 
+	 	 *  "r0" member that comes after the "tables" member.
+		 */
+		up = (struct unw *)(((unsigned long)unw_temp) + 
+			sizeof(struct unw_table *));
+
+		for (i = 0; i < 8; i++)
+			unw.save_order[i] = up->save_order[i];
+		for (i = 0; i < (sizeof(struct unw_frame_info) / 8); i++)
+			unw.sw_off[i] = up->sw_off[i];
+		unw.lru_head = up->lru_head;	
+		unw.lru_tail = up->lru_tail;	
+		for (i = 0; i < UNW_NUM_REGS; i++)
+			unw.preg_index[i] = up->preg_index[i];
+		BCOPY(&up->kernel_table, &unw.kernel_table, 
+			sizeof(struct unw_table));
+
+		FREEBUF(unw_temp);
+	} else {
+		if (!readmem(symbol_value("unw")+ms->unw_tables_offset, 
+	    	    KVADDR, &unw, sizeof(struct unw), "unw", RETURN_ON_ERROR|QUIET)) 
+			return FALSE;
+	}
 
 	if (machdep->flags & UNW_PTREGS) {
 		if (!readmem(symbol_value("unw")+ms->unw_kernel_table_offset+
 			machdep->machspec->unw_tables_offset, 
 	    		KVADDR, &unw.kernel_table, sizeof(struct unw_table), 
 			"unw.kernel_table", RETURN_ON_ERROR|QUIET)) 
+			return FALSE;
 		if (!readmem(symbol_value("unw")+ms->unw_pt_regs_offsets+
 			machdep->machspec->unw_tables_offset, 
 	    		KVADDR, &pt_regs_offsets, sizeof(pt_regs_offsets), 
 			"unw.pt_regs_offsets", RETURN_ON_ERROR|QUIET)) 
-		return FALSE;
+			return FALSE;
 	}
 
 	len = unw.kernel_table.length * sizeof(struct unw_table_entry);
@@ -1588,7 +1643,7 @@ unwind_v3(struct bt_info *bt)
 		CRASHDEBUG_SUSPEND(bt->debug);
 
 	if (!load_unw_table(unw_in_progress ? CLEAR_SCRIPT_CACHE : 0))
-		error(FATAL, "cannot read kernel unw table\n");
+		error(FATAL, "unwind: cannot read kernel unw table\n");
 
 	unw_in_progress = TRUE;
 
@@ -1600,6 +1655,7 @@ unwind_v3(struct bt_info *bt)
         frame = 0;
 
         do {
+restart:
                 unw_get_ip(info, &ip);
                 unw_get_sp(info, &sp);
                 unw_get_bsp(info, &bsp);
@@ -1647,9 +1703,30 @@ unwind_v3(struct bt_info *bt)
 		}
 
 		if (STREQ(name, "start_kernel") || 
+		    STREQ(name, "start_secondary") ||
 		    STREQ(name, "start_kernel_thread"))
                         break;
 
+		/* 
+		 * "init_handler_platform" indicates that this task was
+		 * interrupted by INIT and its stack was switched. 
+		 */
+		if (STREQ(name, "init_handler_platform")) {
+			unw_switch_from_osinit_v1(info, bt);
+			frame++;
+			goto restart;
+		}
+
+		/*
+		 * In some cases, init_handler_platform is inlined into
+		 * ia64_init_handler.  
+		 */
+		if (STREQ(name, "ia64_init_handler")) {
+			unw_switch_from_osinit_v2(info, bt);
+			frame++;
+			goto restart;
+		}
+
                 frame++;
 
         } while (unw_unwind(info) >= 0);
@@ -1752,6 +1829,16 @@ dump_unwind_table(struct unw_table *table)
 	dump_struct("unw_table", (ulong)table, RADIX(16));
 }
 
+static unsigned long 
+get_init_stack_ulong(unsigned long addr) 
+{
+        unsigned long tmp;
+
+        readmem(addr, KVADDR, &tmp, sizeof(unsigned long),
+                "get_init_stack_ulong", FAULT_ON_ERROR);
+
+        return tmp;
+}
 
 static int
 unw_init_from_blocked_task(struct unw_frame_info *info, struct bt_info *bt)
@@ -1759,13 +1846,129 @@ unw_init_from_blocked_task(struct unw_frame_info *info, struct bt_info *bt)
 	ulong sw;
 
 	sw = SWITCH_STACK_ADDR(bt->task);
-	if (!INSTACK(sw, bt))
+	if (!INSTACK(sw, bt) && !ia64_in_init_stack(sw))
 		return FALSE;
 
         unw_init_frame_info(info, bt, sw);
 	return TRUE;
 }
 
+/*
+ * unw_init_from_interruption
+ *   Initialize frame info from specified pt_regs/switch_stack.
+ *
+ *   Similar to unw_init_frame_info() except that:
+ *     - do not use readmem to access stack
+ *       (because stack may be modified by unw_init_from_saved_regs)
+ *     - use ar.ifs and ar.iip instead of ar.pfs and b0, respectively
+ *     - use sof(size-of-frame) of ar.ifs to caluculate bsp,
+ *       instead of sol(size-of-local) of ar.pfs
+ *       (because of cover instruction in kernel minstate save macro)
+ */
+static void
+unw_init_from_interruption(struct unw_frame_info *info, struct bt_info *bt, ulong pt, ulong sw)
+{
+//	unsigned long rbslimit, rbstop, stklimit, stktop, sof, ar_pfs;
+	unsigned long rbslimit, rbstop, stklimit, stktop, sof;
+	ulong t;
+
+	t = bt->task;
+
+	memset(info, 0, sizeof(*info));
+
+	rbslimit = (unsigned long) t + IA64_RBS_OFFSET;
+	rbstop = IA64_GET_STACK_ULONG(sw + OFFSET(switch_stack_ar_bspstore));
+	if (rbstop - (unsigned long) t >= IA64_STK_OFFSET)
+		rbstop = rbslimit;
+
+	stklimit = (unsigned long) t + IA64_STK_OFFSET;
+	stktop   = IA64_GET_STACK_ULONG(pt + offsetof(struct pt_regs, r12));
+	if (stktop <= rbstop)
+		stktop = rbstop;
+
+	info->regstk.limit = rbslimit;
+	info->regstk.top   = rbstop;
+	info->memstk.limit = stklimit;
+	info->memstk.top   = stktop;
+	info->task = (struct task_struct *)bt;
+	info->sw  = (struct switch_stack *)sw;
+	info->sp = info->psp = stktop;
+	info->pr = IA64_GET_STACK_ULONG(sw + OFFSET(switch_stack_pr));
+
+	info->cfm_loc = (unsigned long *) (pt + offsetof(struct pt_regs, cr_ifs));
+	info->unat_loc = (unsigned long *) (pt + offsetof(struct pt_regs, ar_unat));
+	info->pfs_loc = (unsigned long *) (pt + offsetof(struct pt_regs, ar_pfs));
+	/* register stack is covered */
+	sof = IA64_GET_STACK_ULONG(info->cfm_loc) & 0x7f;
+	info->bsp = (unsigned long)
+		ia64_rse_skip_regs((unsigned long *) info->regstk.top, -sof);
+	/* interrupted ip is saved in iip */
+	info->ip = IA64_GET_STACK_ULONG(pt + offsetof(struct pt_regs, cr_iip));
+#if defined(UNWIND_V2) || defined(UNWIND_V3)
+	info->pt = pt;
+#endif
+
+	find_save_locs(info);
+}
+
+/*
+ * unw_switch_from_osinit
+ *   switch back to interrupted context
+ *
+ *   assumption: init_handler_platform() has 3 arguments,
+ *               2nd arg is pt_regs and 3rd arg is switch_stack.
+ */
+static int
+unw_switch_from_osinit_v1(struct unw_frame_info *info, struct bt_info *bt)
+{
+	unsigned long pt, sw;
+	char is_nat;
+
+	/* pt_regs is the 2nd argument of init_handler_platform */
+	if (unw_get_gr(info, 33, &pt, &is_nat)) {
+		fprintf(fp, "gr 33 get error\n");
+		return FALSE;
+	}
+	/* switch_stack is the 3rd argument of init_handler_platform */
+	if (unw_get_gr(info, 34, &sw, &is_nat)) {
+		fprintf(fp, "gr 33 get error\n");
+		return FALSE;
+	}
+
+	unw_init_from_interruption(info, bt, pt, sw);
+	ia64_exception_frame(pt, bt);
+
+	return TRUE;
+}
+
+static int
+unw_switch_from_osinit_v2(struct unw_frame_info *info, struct bt_info *bt)
+{
+	unsigned long pt, sw;
+	char is_nat;
+
+	/* pt_regs is the 1st argument of ia64_init_handler */
+	if (unw_get_gr(info, 32, &pt, &is_nat)) {
+		fprintf(fp, "gr 32 get error\n");
+
+		return FALSE;
+	}
+	/* switch_stack is the 2nd argument of ia64_init_handler */
+	if (unw_get_gr(info, 33, &sw, &is_nat)) {
+		fprintf(fp, "gr 33 get error\n");
+		return FALSE;
+	}
+
+	/* Fix me! */
+	sw = info->psp + 16;
+	pt = sw + STRUCT_SIZE("switch_stack");
+
+	unw_init_from_interruption(info, bt, pt, sw);
+	ia64_exception_frame(pt, bt);
+
+	return TRUE;
+}
+
 static void
 unw_init_frame_info (struct unw_frame_info *info, struct bt_info *bt, ulong sw)
 {
@@ -1806,12 +2009,12 @@ unw_init_frame_info (struct unw_frame_info *info, struct bt_info *bt, ulong sw)
 	info->sw  = (struct switch_stack *)sw;
 	info->sp = info->psp = (unsigned long) (sw + SIZE(switch_stack)) - 16;
         info->cfm_loc = (ulong *)(sw + OFFSET(switch_stack_ar_pfs));
-    	ar_pfs = GET_STACK_ULONG(info->cfm_loc); 
+    	ar_pfs = IA64_GET_STACK_ULONG(info->cfm_loc); 
 	sol = (ar_pfs >> 7) & 0x7f;
 	info->bsp = (unsigned long) 
 		ia64_rse_skip_regs((unsigned long *) info->regstk.top, -sol);
-        info->ip = GET_STACK_ULONG(sw + OFFSET(switch_stack_b0)); 
-        info->pr = GET_STACK_ULONG(sw + OFFSET(switch_stack_pr)); 
+        info->ip = IA64_GET_STACK_ULONG(sw + OFFSET(switch_stack_b0)); 
+        info->pr = IA64_GET_STACK_ULONG(sw + OFFSET(switch_stack_pr)); 
 
 	find_save_locs(info);
 }
@@ -1928,7 +2131,7 @@ unw_unwind (struct unw_frame_info *info)
 		       	info->ip);
 		return -1;
 	}
-	ip = info->ip = GET_STACK_ULONG(info->rp_loc);
+	ip = info->ip = IA64_GET_STACK_ULONG(info->rp_loc);
 	if (ip < GATE_ADDR + PAGE_SIZE) {
 		/*
 		 * We don't have unwind info for the gate page, 
@@ -1952,26 +2155,26 @@ unw_unwind (struct unw_frame_info *info)
 	if ((info->flags & UNW_FLAG_INTERRUPT_FRAME)) {
 #ifdef UNWIND_V1
 		if ((pr & (1UL << pNonSys)) != 0)
-			num_regs = GET_STACK_ULONG(info->cfm_loc) & 0x7f;		/* size of frame */
+			num_regs = IA64_GET_STACK_ULONG(info->cfm_loc) & 0x7f;		/* size of frame */
 		info->pfs_loc =
 			(unsigned long *) (info->sp + 16 + struct_offset(struct pt_regs, ar_pfs));
 #endif
 #ifdef UNWIND_V2
                 info->pt = info->sp + 16;
                 if ((pr & (1UL << pNonSys)) != 0)
-                        num_regs = GET_STACK_ULONG(info->cfm_loc) & 0x7f;               /* size of frame */
+                        num_regs = IA64_GET_STACK_ULONG(info->cfm_loc) & 0x7f;               /* size of frame */
                 info->pfs_loc =
                         (unsigned long *) (info->pt + offsetof(struct pt_regs, ar_pfs));
 #endif
 #ifdef UNWIND_V3
                 info->pt = info->sp + 16;
                 if ((pr & (1UL << pNonSys)) != 0)
-                        num_regs = GET_STACK_ULONG(info->cfm_loc) & 0x7f;               /* size of frame */
+                        num_regs = IA64_GET_STACK_ULONG(info->cfm_loc) & 0x7f;               /* size of frame */
                 info->pfs_loc =
                         (unsigned long *) (info->pt + offsetof(struct pt_regs, ar_pfs));
 #endif
 	} else
-		num_regs = (GET_STACK_ULONG(info->cfm_loc) >> 7) & 0x7f;	/* size of locals */
+		num_regs = (IA64_GET_STACK_ULONG(info->cfm_loc) >> 7) & 0x7f;	/* size of locals */
 	info->bsp = (unsigned long) ia64_rse_skip_regs((unsigned long *) info->bsp, -num_regs);
 	if (info->bsp < info->regstk.limit || info->bsp > info->regstk.top) {
 		error(INFO, "unwind: bsp (0x%lx) out of range [0x%lx-0x%lx]\n",
@@ -1981,7 +2184,8 @@ unw_unwind (struct unw_frame_info *info)
 
 	/* restore the sp: */
 	info->sp = info->psp;
-	if (info->sp < info->memstk.top || info->sp > info->memstk.limit) {
+	if ((info->sp < info->memstk.top || info->sp > info->memstk.limit)
+		&& !ia64_in_init_stack(info->sp)) {
 		error(INFO, "unwind: sp (0x%lx) out of range [0x%lx-0x%lx]\n",
 			info->sp, info->memstk.top, info->memstk.limit);
 		return -1;
@@ -2090,7 +2294,7 @@ run_script (struct unw_script *script, struct unw_frame_info *state)
 				break;
 			}
 #endif
-			s[dst] = GET_STACK_ULONG(s[val]);
+			s[dst] = IA64_GET_STACK_ULONG(s[val]);
 			break;
 		}
 	}
@@ -2413,8 +2617,9 @@ lookup(struct unw_table *table, unsigned long rel_ip)
 	        if (!readmem((ulong)table->array, KVADDR, loc_array, 
 		    table->length * sizeof(struct unw_table_entry),
 	            "module unw_table_entry array", RETURN_ON_ERROR|QUIET)) {
-	                error(WARNING, 
-			    	"cannot read module unw_table_entry array\n");
+			if (IS_MODULE_VADDR(table->segment_base + rel_ip))
+	                	error(WARNING, 
+		    	          "cannot read module unw_table_entry array\n");
 	                return 0;
 	        }
 		array = loc_array;
