@@ -27,7 +27,7 @@
  *
  * 09/28/00  ---    Transition to CVS version control
  *
- * CVS: $Revision: 1.74 $ $Date: 2004/06/22 17:57:05 $
+ * CVS: $Revision: 1.78 $ $Date: 2005/02/14 21:59:44 $
  */
 
 #ifdef X86
@@ -65,7 +65,7 @@
  * any improvements or extensions that they make and grant Carnegie the
  * rights to redistribute these changes.
  *
- *	$Id: x86.c,v 1.74 2004/06/22 17:57:05 anderson Exp $
+ *	$Id: x86.c,v 1.78 2005/02/14 21:59:44 anderson Exp $
  */
 #include "defs.h"
 
@@ -1350,7 +1350,8 @@ try_closest:
         if (strlen(buf)) {
                 if (retries) {
                         p = strstr(buf, ": ");
-			*p = NULLCHAR;
+			if (p)
+				*p = NULLCHAR;
                 }
                 fprintf(fp, "    %s\n", buf);
         } else {
@@ -1728,6 +1729,7 @@ x86_init(int when)
 		STRUCT_SIZE_INIT(cpuinfo_x86, "cpuinfo_x86");
 		STRUCT_SIZE_INIT(e820map, "e820map");
 		STRUCT_SIZE_INIT(e820entry, "e820entry");
+		STRUCT_SIZE_INIT(irq_ctx, "irq_ctx");
 		MEMBER_OFFSET_INIT(e820map_nr_map, "e820map", "nr_map");
 		MEMBER_OFFSET_INIT(e820entry_addr, "e820entry", "addr");
 		MEMBER_OFFSET_INIT(e820entry_size, "e820entry", "size");
@@ -1738,6 +1740,8 @@ x86_init(int when)
 		else
 			machdep->nr_irqs = 224;  /* NR_IRQS */
 		machdep->hz = HZ;
+		if (THIS_KERNEL_VERSION >= LINUX(2,6,0))
+			machdep->hz = 1000;
 		break;
 
 	case POST_INIT:
@@ -2935,6 +2939,25 @@ x86_dis_filter(ulong vaddr, char *inbuf)
 			value_to_symstr(value, buf2, pc->output_radix));
 
 		sprintf(p1, buf1);
+	} else if (STREQ(argv[argc-2], "call") && 
+	    hexadecimal(argv[argc-1], 0)) {
+		/* 
+		 *  Update module code of the form:
+		 *
+		 *    call   0xe081e1e0
+		 *
+		 *  to show a bracketed direct call target.
+		 */
+		p1 = &LASTCHAR(inbuf);
+
+                if (extract_hex(argv[argc-1], &value, NULLCHAR, TRUE)) {
+                	sprintf(buf1, " <%s>\n",
+				value_to_symstr(value, buf2,
+                                pc->output_radix));
+                        if (IS_MODULE_VADDR(value) &&
+                            !strstr(buf2, "+"))
+                                sprintf(p1, buf1);
+		}
 	}
 
 	if (CRASHDEBUG(1))
