@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * CVS: $Revision: 1.34 $ $Date: 2004/06/15 19:08:09 $
+ * CVS: $Revision: 1.41 $ $Date: 2005/02/14 21:59:44 $
  */
 #include "defs.h"
 
@@ -64,6 +64,7 @@ void x86_64_compiler_warning_stub(void);
 static void x86_64_init_kernel_pgd(void);
 static void x86_64_cpu_pda_init(void);
 static void x86_64_ist_init(void);
+static void x86_64_post_init(void);
 
 
 struct machine_specific x86_64_machine_specific = { 0 };
@@ -167,10 +168,12 @@ x86_64_init(int when)
 		machdep->vmalloc_start = x86_64_vmalloc_start;
 		machdep->dump_irq = x86_64_dump_irq;
 		machdep->hz = HZ;
-
+		if (THIS_KERNEL_VERSION >= LINUX(2,6,0))
+			machdep->hz = 1000;
 		break;
 
 	case POST_INIT:
+		x86_64_post_init();
 		break;
 	}
 }
@@ -374,10 +377,19 @@ x86_64_cpu_pda_init(void)
 	 *  the address of &boot_cpu_stack[0].
 	 */
 	sp = value_search(machdep->machspec->stkinfo.ibase[0], &offset);
-	if (!sp || offset || !STREQ(sp->name, "boot_cpu_stack"))
-		error(WARNING, 
-		    "cpu 0 IRQ stack: %lx\n         boot_cpu_stack: %lx\n\n", 
-			machdep->machspec->stkinfo.ibase[0], sp->value);
+	if (!sp || offset || !STREQ(sp->name, "boot_cpu_stack")) {
+		if (symbol_value("boot_cpu_stack")) {
+			error(WARNING, 
+		       "cpu 0 IRQ stack: %lx\n         boot_cpu_stack: %lx\n\n",
+				machdep->machspec->stkinfo.ibase[0], 
+				symbol_value("boot_cpu_stack"));
+			if (!machdep->machspec->stkinfo.ibase[0])
+				machdep->machspec->stkinfo.ibase[0] = 
+					symbol_value("boot_cpu_stack");
+		} else
+			error(WARNING, 
+	 	     "boot_cpu_stack: symbol does not exist in this kernel!\n");
+	}
 
 	kt->cpus = cpus;
 	if (kt->cpus > 1)
@@ -401,16 +413,37 @@ x86_64_ist_init(void)
 	struct syment *sp;
 
         ms = machdep->machspec;
-	init_tss = symbol_value("init_tss");
+	if (symbol_exists("init_tss")) {
+		init_tss = symbol_value("init_tss");
+	
+		for (c = cpus = 0; c < NR_CPUS; c++) {
+			vaddr = init_tss + (c * SIZE(tss_struct)) +
+				OFFSET(tss_struct_ist); 
+			readmem(vaddr, KVADDR, &ms->stkinfo.ebase[c][0], 
+				sizeof(ulong) * 7, "tss_struct ist array", 
+				FAULT_ON_ERROR);
+			if (ms->stkinfo.ebase[c][0] == 0)
+				break;
+		}
+	} else if (symbol_exists("per_cpu__init_tss")) {
+		for (c = 0; c < NR_CPUS; c++) {
+                	if ((kt->flags & SMP) && (kt->flags & PER_CPU_OFF)) {
+				if (kt->__per_cpu_offset[c] == 0)
+					break;
+                		vaddr = symbol_value("per_cpu__init_tss") +
+                                        kt->__per_cpu_offset[c];
+			} else 
+				vaddr = symbol_value("per_cpu__init_tss");
 
-	for (c = cpus = 0; c < NR_CPUS; c++) {
-		vaddr = init_tss + (c * SIZE(tss_struct)) +
-			OFFSET(tss_struct_ist); 
-		readmem(vaddr, KVADDR, &ms->stkinfo.ebase[c][0], 
-			sizeof(ulong) * 7, "tss_struct ist array", 
-			FAULT_ON_ERROR);
-		if (ms->stkinfo.ebase[c][0] == 0)
-			break;
+			vaddr += OFFSET(tss_struct_ist);
+
+                        readmem(vaddr, KVADDR, &ms->stkinfo.ebase[c][0],
+                                sizeof(ulong) * 7, "tss_struct ist array",
+                                FAULT_ON_ERROR);
+
+                        if (ms->stkinfo.ebase[c][0] == 0)
+                                break;
+		}
 	}
 
 	if (ms->stkinfo.ebase[0][0] && ms->stkinfo.ebase[0][1])
@@ -436,10 +469,68 @@ x86_64_ist_init(void)
 	 *  located at: &boot_exception_stacks[0]
 	 */
         sp = value_search(ms->stkinfo.ebase[0][0], &offset);
-       	if (!sp || offset || !STREQ(sp->name, "boot_exception_stacks"))
-                error(WARNING,
+       	if (!sp || offset || !STREQ(sp->name, "boot_exception_stacks")) {
+		if (symbol_value("boot_exception_stacks")) {
+                	error(WARNING,
     "cpu 0 first exception stack: %lx\n         boot_exception_stacks: %lx\n\n",
-                        ms->stkinfo.ebase[0][0], sp->value);
+                        	ms->stkinfo.ebase[0][0], 
+				symbol_value("boot_exception_stacks"));
+			if (!ms->stkinfo.ebase[0][0])
+				ms->stkinfo.ebase[0][0] = 
+					symbol_value("boot_exception_stacks");
+		} else 
+			error(WARNING, 
+	      "boot_exception_stacks: symbol does not exist in this kernel!\n");
+	}
+}
+
+static void 
+x86_64_post_init(void)
+{ 
+        int c, i, clues;
+        struct machine_specific *ms;
+	ulong *up;
+	struct syment *spt, *spc;
+	ulong offset;
+
+	/*
+	 *  Check whether each cpu was stopped by an NMI.
+	 */
+        ms = machdep->machspec;
+
+        for (c = 0; c < kt->cpus; c++) {
+                if (ms->stkinfo.ebase[c][NMI_STACK] == 0)
+                        break;
+
+                if (!readmem(ms->stkinfo.ebase[c][NMI_STACK], 
+		    KVADDR, ms->irqstack,
+		    ms->stkinfo.esize,
+                    "NMI exception stack contents", 
+		    RETURN_ON_ERROR|QUIET)) 
+			continue;
+
+       		for (i = clues = 0; i < (ms->stkinfo.esize)/sizeof(ulong); i++){
+                	up = (ulong *)(&ms->irqstack[i*sizeof(ulong)]);
+
+                	if (!is_kernel_text(*up) ||
+                            !(spt = value_search(*up, &offset)))
+				continue;
+
+			if (STREQ(spt->name, "try_crashdump") ||
+			    STREQ(spt->name, "die_nmi")) 
+				clues++;
+
+                    	if ((STREQ(spt->name, "nmi_watchdog_tick") ||
+                     	     STREQ(spt->name, "default_do_nmi"))) {
+                        	spc = x86_64_function_called_by((*up)-5);
+                        	if (spc && STREQ(spc->name, "die_nmi"))
+                                	clues += 2;
+			}
+		}
+
+		if (clues >= 2) 
+			kt->cpu_flags[c] |= NMI;
+        }
 }
 
 /*
@@ -1088,7 +1179,8 @@ x86_64_print_stack_entry(struct bt_info *bt, FILE *ofp, int level,
 		return BACKTRACE_ENTRY_DISPLAYED;
 	}
 
-	if (!offset && !(bt->flags & BT_EXCEPTION_FRAME)) { 
+	if (!offset && !(bt->flags & BT_EXCEPTION_FRAME) &&
+	    !(bt->flags & BT_START)) { 
 		if (STREQ(name, "child_rip")) {
 			if (symbol_exists("kernel_thread"))
 				name = "kernel_thread";
@@ -1124,6 +1216,8 @@ x86_64_print_stack_entry(struct bt_info *bt, FILE *ofp, int level,
 
 	if (bt->flags & BT_EXCEPTION_FRAME)
 		rsp = bt->stkptr;
+	else if (bt->flags & BT_START)
+		rsp = bt->stkptr;
 	else
 		rsp = bt->stackbase + (stkindex * sizeof(long));
 
@@ -1280,6 +1374,10 @@ x86_64_low_budget_back_trace_cmd(struct bt_info *bt_in)
 	irq_eframe = 0;
 	bt->call_target = NULL;
 	rsp = bt->stkptr;
+	if (!rsp) {
+		error(INFO, "cannot determine starting stack pointer\n");
+		return;
+	}
 	ms = machdep->machspec;
 	if (BT_REFERENCE_CHECK(bt))
 		ofp = pc->nullfp;
@@ -1293,7 +1391,12 @@ x86_64_low_budget_back_trace_cmd(struct bt_info *bt_in)
 			STREQ(closest_symbol(bt->instptr), "thread_return") ?
 			" (schedule)" : "",
 			bt->instptr);
-        }
+        } else if (bt->flags & BT_START) {
+                x86_64_print_stack_entry(bt, ofp, level,
+                        0, bt->instptr);
+		bt->flags &= ~BT_START;
+		level++;
+	}
 
 
         if ((estack = x86_64_in_exception_stack(bt))) {
@@ -1363,7 +1466,8 @@ in_exception_stack:
 			bt->stackbuf + (bt->stacktop - bt->stackbase) - 
 			SIZE(pt_regs), bt, ofp);
 
-		fprintf(fp, "--- <exception stack> ---\n");
+		if (!BT_REFERENCE_CHECK(bt))
+			fprintf(fp, "--- <exception stack> ---\n");
 
                 /* 
 		 *  stack = (unsigned long *) estack_end[-2]; 
@@ -1371,13 +1475,24 @@ in_exception_stack:
 		up = (ulong *)(&bt->stackbuf[bt->stacktop - bt->stackbase]);
 		up -= 2;
 		rsp = bt->stkptr = *up;
-		up -= 5;
+		up -= 3;
 		bt->instptr = *up;  
 		if (cs & 3)
 			done = TRUE;   /* user-mode exception */
 		else
 			done = FALSE;  /* kernel-mode exception */
 		bt->frameptr = 0;
+
+		/*
+		 *  Print the return values from the estack end.
+		 */
+		if (!done) {
+                	bt->flags |= BT_START;
+                	x86_64_print_stack_entry(bt, ofp, level,
+                        	0, bt->instptr);
+                	bt->flags &= ~BT_START;
+			level++;
+		}
 	}
 
 	/*
@@ -1426,7 +1541,8 @@ in_exception_stack:
                         }
                 }
 
-                fprintf(fp, "--- <IRQ stack> ---\n");
+		if (!BT_REFERENCE_CHECK(bt))
+                	fprintf(fp, "--- <IRQ stack> ---\n");
 
                 /*
 		 *  stack = (unsigned long *) (irqstack_end[-1]);
@@ -1910,6 +2026,7 @@ x86_64_get_dumpfile_stack_frame(struct bt_info *bt_in, ulong *rip, ulong *rsp)
 	int panic_task;
         int i, panic, stage;
         char *sym;
+	struct syment *sp;
         ulong *up;
 	struct bt_info bt_local, *bt;
         struct machine_specific *ms;
@@ -1957,12 +2074,29 @@ next_stack:
                 sym = closest_symbol(*up);
 
                 if (STREQ(sym, "netconsole_netdump") || 
-		    STREQ(sym, "disk_dump")) {
+		    STREQ(sym, "netpoll_start_netdump") ||
+		    STREQ(sym, "start_disk_dump") ||
+		    STREQ(sym, "disk_dump") ||
+		    STREQ(sym, "try_crashdump")) {
                         *rip = *up;
                         *rsp = bt->stackbase + ((char *)(up) - bt->stackbuf);
                         return;
                 }
 
+                if ((stage == 2) && 
+                    (STREQ(sym, "nmi_watchdog_tick") ||
+                     STREQ(sym, "default_do_nmi"))) {
+			sp = x86_64_function_called_by((*up)-5);
+			if (!sp || !STREQ(sp->name, "die_nmi")) 
+				continue;
+                        *rip = *up;
+                        *rsp = bt->stackbase + ((char *)(up) - bt->stackbuf);
+			bt_in->flags |= BT_START;
+			*rip = symbol_value("die_nmi");
+			*rsp = (*rsp) - (7*sizeof(ulong));
+                        return;
+                }
+
                 if (STREQ(sym, "panic")) {
                         *rip = *up;
                         *rsp = bt->stackbase + ((char *)(up) - bt->stackbuf);
@@ -2020,7 +2154,19 @@ skip_stage:
 		stage = 1;
 		goto next_stack;
 
+        /*
+         *  Check the NMI exception stack.
+         */
 	case 1:
+		bt->stackbase = ms->stkinfo.ebase[bt->tc->processor][NMI_STACK];
+		bt->stacktop = ms->stkinfo.ebase[bt->tc->processor][NMI_STACK] +
+                       ms->stkinfo.esize;
+		bt->stackbuf = ms->irqstack;
+		alter_stackbuf(bt);
+		stage = 2;
+		goto next_stack;
+
+	case 2:
 		break;
 	}
 
@@ -2235,7 +2381,27 @@ x86_64_dis_filter(ulong vaddr, char *inbuf)
 			value_to_symstr(value, buf2, pc->output_radix));
 
 		sprintf(p1, buf1);
-	}
+	
+        } else if (STREQ(argv[argc-2], "callq") &&
+            hexadecimal(argv[argc-1], 0)) {
+            	/*
+             	 *  Update module code of the form:
+             	 *
+             	 *    callq  0xffffffffa0017aa0
+	      	 *
+             	 *  to show a bracketed direct call target.
+             	 */
+                p1 = &LASTCHAR(inbuf);
+
+                if (extract_hex(argv[argc-1], &value, NULLCHAR, TRUE)) {
+                        sprintf(buf1, " <%s>\n",
+                                value_to_symstr(value, buf2,
+                                pc->output_radix));
+                        if (IS_MODULE_VADDR(value) &&
+                            !strstr(buf2, "+"))
+                                sprintf(p1, buf1);
+                }
+        }
 
 	console("    %s", inbuf);
 
@@ -2253,6 +2419,9 @@ x86_64_get_smp_cpus(void)
 	char *cpu_pda_buf;
 	ulong level4_pgt;
 
+	if (!VALID_SIZE(x8664_pda))
+		return 1;
+
 	cpu_pda_buf = GETBUF(SIZE(x8664_pda));
 
 	for (i = cpus = 0; i < NR_CPUS; i++) {
